@@ -53,6 +53,9 @@ static ParsedLines parse_lines(StringView data)
         pos = eol + 1;
     }
 
+    if (res.lines.empty())
+        res.lines.emplace_back(StringData::create({"\n"}));
+
     return res;
 }
 
@@ -79,9 +82,6 @@ Buffer::Buffer(String name, Flags flags, StringView data,
 {
     ParsedLines parsed_lines = parse_lines(data);
 
-    if (parsed_lines.lines.empty())
-        parsed_lines.lines.emplace_back(StringData::create({"\n"}));
-
     #ifdef KAK_DEBUG
     for (auto& line : parsed_lines.lines)
         kak_assert(not (line->length == 0) and
@@ -245,9 +245,6 @@ void Buffer::reload(StringView data, timespec fs_timestamp)
 {
     ParsedLines parsed_lines = parse_lines(data);
 
-    if (parsed_lines.lines.empty())
-        parsed_lines.lines.emplace_back(StringData::create({"\n"}));
-
     const bool record_undo = not (m_flags & Flags::NoUndo);
 
     commit_undo_group();
@@ -484,7 +481,7 @@ BufferCoord Buffer::do_insert(BufferCoord pos, StringView content)
                             : BufferCoord{ last_line, m_lines[last_line].length() - suffix.length() };
 
     m_changes.push_back({ Change::Insert, pos, end });
-    return pos;
+    return end;
 }
 
 BufferCoord Buffer::do_erase(BufferCoord begin, BufferCoord end)
@@ -688,7 +685,7 @@ void Buffer::on_option_changed(const Option& option)
             m_flags &= ~Flags::ReadOnly;
     }
     run_hook_in_own_context(Hook::BufSetOption,
-                            format("{}={}", option.name(), option.get_as_string(Quoting::Kakoune)));
+                            format("{}={}", option.name(), option.get_as_string(Quoting::Raw)));
 }
 
 void Buffer::run_hook_in_own_context(Hook hook, StringView param, String client_name)
@@ -719,12 +716,16 @@ String Buffer::debug_description() const
             return sizeof(history) + history.undo_group.size() * sizeof(Modification) + s;
         }) + m_changes.size() * sizeof(Change);
 
-    return format("{}\nFlags: {}{}{}{}\nUsed mem: content={} additional={}\n",
+    return format("{}\nFlags: {}{}{}{}{}{}{}{}\nUsed mem: content={} additional={}\n",
                   display_name(),
                   (m_flags & Flags::File) ? "File (" + name() + ") " : "",
                   (m_flags & Flags::New) ? "New " : "",
                   (m_flags & Flags::Fifo) ? "Fifo " : "",
                   (m_flags & Flags::NoUndo) ? "NoUndo " : "",
+                  (m_flags & Flags::NoHooks) ? "NoHooks " : "",
+                  (m_flags & Flags::Debug) ? "Debug " : "",
+                  (m_flags & Flags::ReadOnly) ? "ReadOnly " : "",
+                  is_modified() ? "Modified " : "",
                   content_size, additional_size);
 }
 
@@ -809,7 +810,8 @@ UnitTest test_buffer{[]()
 UnitTest test_undo{[]()
 {
     Buffer buffer("test", Buffer::Flags::None, "allo ?\nmais que fais la police\n hein ?\n youpi\n");
-    auto pos = buffer.insert(buffer.end_coord(), "kanaky\n"); // change 1
+    auto pos = buffer.end_coord();
+    buffer.insert(pos, "kanaky\n");                           // change 1
     buffer.commit_undo_group();
     buffer.erase(pos, buffer.end_coord());                    // change 2
     buffer.commit_undo_group();
