@@ -5,6 +5,7 @@
 #include "buffer_utils.hh"
 #include "context.hh"
 #include "flags.hh"
+#include "file.hh"
 #include "optional.hh"
 #include "option_types.hh"
 #include "ranges.hh"
@@ -39,6 +40,36 @@ void CommandManager::register_command(String command_name,
                                  std::move(completer) };
 }
 
+bool CommandManager::module_defined(StringView module_name) const
+{
+    return m_modules.find(module_name) != m_modules.end();
+}
+
+void CommandManager::register_module(String module_name, String commands)
+{
+    auto module = m_modules.find(module_name);
+    if (module != m_modules.end() and module->value.loaded)
+        throw runtime_error{format("module already loaded: '{}'", module_name)};
+
+    m_modules[module_name] = { false, std::move(commands) };
+}
+
+void CommandManager::load_module(StringView module_name, Context& context)
+{
+    auto module = m_modules.find(module_name);
+    if (module == m_modules.end())
+        throw runtime_error{format("no such module: '{}'", module_name)};
+    if (module->value.loaded)
+        return;
+
+    module->value.loaded = true;
+    Context empty_context{Context::EmptyContextFlag{}};
+    execute(module->value.commands, empty_context);
+    module->value.commands.clear();
+
+    context.hooks().run_hook(Hook::ModuleLoaded, module_name, context);
+}
+
 struct parse_error : runtime_error
 {
     parse_error(StringView error)
@@ -121,11 +152,11 @@ QuotedResult parse_quoted_balanced(Reader& reader, char opening_delimiter,
         {
             auto content = reader.substr_from(start);
             ++reader.pos;
-            return {content.str(), true};
+            return {String{String::NoCopy{}, content}, true};
         }
         ++reader.pos;
     }
-    return {reader.substr_from(start).str(), false};
+    return {String{String::NoCopy{}, reader.substr_from(start)}, false};
 }
 
 String parse_unquoted(Reader& reader)
@@ -168,6 +199,8 @@ Token::Type token_type(StringView type_name, bool throw_on_invalid)
         return Token::Type::ValExpand;
     else if (type_name == "arg")
         return Token::Type::ArgExpand;
+    else if (type_name == "file")
+        return Token::Type::FileExpand;
     else if (throw_on_invalid)
         throw parse_error{format("unknown expand '{}'", type_name)};
     else
@@ -253,7 +286,7 @@ Token parse_percent_token(Reader& reader, bool throw_on_unterminated)
 
 auto expand_option(Option& opt, std::true_type)
 {
-    return opt.get_as_string(Quoting::Kakoune);
+    return opt.get_as_string(Quoting::Raw);
 }
 
 auto expand_option(Option& opt, std::false_type)
@@ -327,6 +360,8 @@ expand_token(const Token& token, const Context& context, const ShellContext& she
             throw runtime_error("invalid argument index");
         return {arg < params.size() ? params[arg] : String{}};
     }
+    case Token::Type::FileExpand:
+        return {read_file(content)};
     case Token::Type::RawEval:
         return {expand(content, context, shell_context)};
     case Token::Type::Raw:
@@ -589,6 +624,13 @@ Completions CommandManager::complete_command_name(const Context& context, String
     return {0, query.length(), Kakoune::complete(query, query.length(), concatenated(commands, aliases))};
 }
 
+Completions CommandManager::complete_module_name(StringView query) const
+{
+    return {0, query.length(),
+            Kakoune::complete(query, query.length(), m_modules | filter([](auto&& item) { return not item.value.loaded; })
+                                                               | transform(&ModuleMap::Item::key))};
+}
+
 Completions CommandManager::complete(const Context& context,
                                      CompletionFlags flags,
                                      StringView command_line,
@@ -684,7 +726,7 @@ Completions CommandManager::complete(const Context& context,
             context, flags, params, tokens.size() - 2,
             cursor_pos_in_token), start);
 
-        if (not completions.quoted and token.type == Token::Type::Raw)
+        if (not (completions.flags & Completions::Flags::Quoted) and token.type == Token::Type::Raw)
         {
             for (auto& c : completions.candidates)
                 c = (not c.empty() and contains("%'\"", c[0]) ? "\\" : "") + escape(c, "; \t", '\\');
