@@ -86,7 +86,7 @@ String real_path(StringView filename)
                 return res;
 
             StringView dir = res;
-            if (dir.substr(dir.length()-1_byte, 1_byte) == "/")
+            while (not dir.empty() and dir.back() == '/')
                 dir = dir.substr(0_byte, dir.length()-1_byte);
             return format("{}/{}", dir, non_existing);
         }
@@ -115,7 +115,10 @@ String compact_path(StringView filename)
     if (prefix_match(real_filename, real_cwd))
         return real_filename.substr(real_cwd.length()).str();
 
-    const StringView home = homedir();
+    StringView home = homedir();
+    while (not home.empty() and home.back() == '/')
+        home = home.substr(0_byte, home.length()-1_byte);
+
     if (not home.empty())
     {
         ByteCount home_len = home.length();
@@ -145,6 +148,7 @@ StringView homedir()
 
 bool fd_readable(int fd)
 {
+    kak_assert(fd >= 0);
     fd_set  rfds;
     FD_ZERO(&rfds);
     FD_SET(fd, &rfds);
@@ -155,6 +159,7 @@ bool fd_readable(int fd)
 
 bool fd_writable(int fd)
 {
+    kak_assert(fd >= 0);
     fd_set  wfds;
     FD_ZERO(&wfds);
     FD_SET(fd, &wfds);
@@ -257,7 +262,53 @@ void write(int fd, StringView data)
     }
 }
 
-void write_buffer_to_fd(Buffer& buffer, int fd, bool sync)
+void write_to_file(StringView filename, StringView data)
+{
+    const int fd = open(filename.zstr(), O_CREAT | O_WRONLY | O_TRUNC, 0644);
+    if (fd == -1)
+        throw file_access_error(filename, strerror(errno));
+    auto close_fd = on_scope_end([fd]{ close(fd); });
+    write(fd, data);
+}
+
+struct BufferedWriter
+{
+    BufferedWriter(int fd)
+      : m_fd{fd}, m_exception_count{std::uncaught_exceptions()} {}
+
+    ~BufferedWriter() noexcept(false)
+    {
+        if (m_pos != 0 and m_exception_count == std::uncaught_exceptions())
+            Kakoune::write(m_fd, {m_buffer, m_pos});
+    }
+
+    void write(StringView data)
+    {
+        while (not data.empty())
+        {
+            const ByteCount length = data.length();
+            const ByteCount write_len = std::min(length, size - m_pos);
+            memcpy(m_buffer + (int)m_pos, data.data(), (int)write_len);
+            m_pos += write_len;
+            if (m_pos == size)
+            {
+                Kakoune::write(m_fd, {m_buffer, size});
+                m_pos = 0;
+            }
+            data = data.substr(write_len);
+        }
+    }
+
+private:
+    static constexpr ByteCount size = 4096;
+    int m_fd;
+    int m_exception_count;
+    ByteCount m_pos = 0;
+    char m_buffer[(int)size];
+};
+
+
+void write_buffer_to_fd(Buffer& buffer, int fd)
 {
     auto eolformat = buffer.options()["eolformat"].get<EolFormat>();
     StringView eoldata;
@@ -266,52 +317,79 @@ void write_buffer_to_fd(Buffer& buffer, int fd, bool sync)
     else
         eoldata = "\n";
 
+
+    BufferedWriter writer{fd};
     if (buffer.options()["BOM"].get<ByteOrderMark>() == ByteOrderMark::Utf8)
-        if (::write(fd, "\xEF\xBB\xBF", 3) < 0)
-            throw runtime_error(format("unable to write data to the buffer (fd: {}; errno: {})", fd, ::strerror(errno)));
+        writer.write("\xEF\xBB\xBF");
 
     for (LineCount i = 0; i < buffer.line_count(); ++i)
     {
         // end of lines are written according to eolformat but always
         // stored as \n
         StringView linedata = buffer[i];
-        write(fd, linedata.substr(0, linedata.length()-1));
-        write(fd, eoldata);
+        writer.write(linedata.substr(0, linedata.length()-1));
+        writer.write(eoldata);
     }
-
-    if (sync)
-        ::fsync(fd);
 }
 
-void write_buffer_to_file(Buffer& buffer, StringView filename, bool force, bool sync)
+int open_temp_file(StringView filename, char (&buffer)[PATH_MAX])
 {
-    struct stat st;
-    auto zfilename = filename.zstr();
+    String path = real_path(filename);
+    auto [dir,file] = split_path(path);
 
-    if (force)
+    if (dir.empty())
+        format_to(buffer, ".{}.kak.XXXXXX", file);
+    else
+        format_to(buffer, "{}/.{}.kak.XXXXXX", dir, file);
+
+    return mkstemp(buffer);
+}
+
+int open_temp_file(StringView filename)
+{
+    char buffer[PATH_MAX];
+    return open_temp_file(filename, buffer);
+}
+
+void write_buffer_to_file(Buffer& buffer, StringView filename,
+                          WriteMethod method, WriteFlags flags)
+{
+    auto zfilename = filename.zstr();
+    struct stat st;
+
+    bool replace = method == WriteMethod::Replace;
+    bool force = flags & WriteFlags::Force;
+
+    if ((replace or force) and ::stat(zfilename, &st) != 0)
     {
-        if (::stat(zfilename, &st) == 0)
-        {
-            if (::chmod(zfilename, st.st_mode | S_IWUSR) < 0)
-                throw runtime_error("unable to change file permissions");
-        }
-        else
-            force = false;
+        force = false;
+        replace = false;
     }
+
+    if (force and ::chmod(zfilename, st.st_mode | S_IWUSR) < 0)
+        throw runtime_error("unable to change file permissions");
+
     auto restore_mode = on_scope_end([&]{
-        if (force and ::chmod(zfilename, st.st_mode) < 0)
+        if ((force or replace) and ::chmod(zfilename, st.st_mode) < 0)
             throw runtime_error("unable to restore file permissions");
     });
 
-    int fd = open(zfilename, O_CREAT | O_WRONLY | O_TRUNC, 0644);
+    char temp_filename[PATH_MAX];
+    const int fd = replace ? open_temp_file(filename, temp_filename)
+                           : open(zfilename, O_CREAT | O_WRONLY | O_TRUNC, 0644);
     if (fd == -1)
         throw file_access_error(filename, strerror(errno));
 
     {
         auto close_fd = on_scope_end([fd]{ close(fd); });
-        write_buffer_to_fd(buffer, fd, sync);
+        write_buffer_to_fd(buffer, fd);
+        if (flags & WriteFlags::Sync)
+            ::fsync(fd);
     }
 
+    if (replace and rename(temp_filename, zfilename) != 0)
+        throw runtime_error("replacing file failed");
+
     if ((buffer.flags() & Buffer::Flags::File) and
         real_path(filename) == real_path(buffer.name()))
         buffer.notify_saved();
@@ -319,17 +397,7 @@ void write_buffer_to_file(Buffer& buffer, StringView filename, bool force, bool
 
 void write_buffer_to_backup_file(Buffer& buffer)
 {
-    String path = real_path(buffer.name());
-    StringView dir, file;
-    std::tie(dir,file) = split_path(path);
-
-    char pattern[PATH_MAX];
-    if (dir.empty())
-        format_to(pattern, ".{}.kak.XXXXXX", file);
-    else
-        format_to(pattern, "{}/.{}.kak.XXXXXX", dir, file);
-
-    int fd = mkstemp(pattern);
+    const int fd = open_temp_file(buffer.name());
     if (fd >= 0)
     {
         write_buffer_to_fd(buffer, fd);
@@ -440,8 +508,7 @@ CandidateList complete_filename(StringView prefix, const Regex& ignored_regex,
                                 ByteCount cursor_pos, FilenameFlags flags)
 {
     prefix = prefix.substr(0, cursor_pos);
-    StringView dirname, fileprefix;
-    std::tie(dirname, fileprefix) = split_path(prefix);
+    auto [dirname, fileprefix] = split_path(prefix);
     auto parsed_dirname = parse_filename(dirname);
 
     const bool check_ignored_regex = not ignored_regex.empty() and
@@ -469,8 +536,7 @@ CandidateList complete_filename(StringView prefix, const Regex& ignored_regex,
 CandidateList complete_command(StringView prefix, ByteCount cursor_pos)
 {
     String real_prefix = parse_filename(prefix.substr(0, cursor_pos));
-    StringView dirname, fileprefix;
-    std::tie(dirname, fileprefix) = split_path(real_prefix);
+    auto [dirname, fileprefix] = split_path(real_prefix);
 
     if (not dirname.empty())
     {
