@@ -751,10 +751,13 @@ class Prompt : public InputMode
 public:
     Prompt(InputHandler& input_handler, StringView prompt,
            String initstr, String emptystr, Face face, PromptFlags flags,
-           Completer completer, PromptCallback callback)
-        : InputMode(input_handler), m_prompt(prompt.str()), m_prompt_face(face),
+           char history_register, PromptCompleter completer, PromptCallback callback)
+        : InputMode(input_handler), m_callback(std::move(callback)), m_completer(std::move(completer)),
+          m_prompt(prompt.str()), m_prompt_face(face),
           m_empty_text{std::move(emptystr)},
-          m_flags(flags), m_completer(std::move(completer)), m_callback(std::move(callback)),
+          m_line_editor{context().faces()}, m_flags(flags),
+          m_history{RegisterManager::instance()[history_register]},
+          m_current_history{m_history.get(context()).size()},
           m_auto_complete{context().options()["autocomplete"].get<AutoComplete>() & AutoComplete::Prompt},
           m_idle_timer{TimePoint::max(), context().flags() & Context::Flags::Draft ?
                            Timer::Callback{} : [this](Timer&) {
@@ -766,22 +769,28 @@ public:
                                m_line_changed = false;
                            }
                            context().hooks().run_hook(Hook::PromptIdle, "", context());
-                       }},
-          m_line_editor{context().faces()}
+                       }}
     {
-        m_history_it = ms_history[m_prompt].end();
         m_line_editor.reset(std::move(initstr), m_empty_text);
     }
 
     void on_key(Key key) override
     {
-        History& history = ms_history[m_prompt];
         const String& line = m_line_editor.line();
 
         if (key == Key::Return)
         {
+            if ((m_completions.flags & Completions::Flags::Menu) and
+                m_current_completion == -1 and
+                not m_completions.candidates.empty())
+            {
+                const String& completion = m_completions.candidates.front();
+                m_line_editor.insert_from(line.char_count_to(m_completions.start),
+                                          completion);
+            }
+
             if (not context().history_disabled())
-                history_push(history, line);
+                history_push(line);
             context().print_status(DisplayLine{});
             if (context().has_client())
                 context().client().menu_hide();
@@ -798,7 +807,7 @@ public:
         else if (key == Key::Escape or key == ctrl('c'))
         {
             if (not context().history_disabled())
-                history_push(history, line);
+                history_push(line);
             context().print_status(DisplayLine{});
             if (context().has_client())
                 context().client().menu_hide();
@@ -844,22 +853,25 @@ public:
         }
         else if (key == Key::Up or key == ctrl('p'))
         {
-            if (m_history_it != history.begin())
+            if (m_current_history != 0)
             {
-                if (m_history_it == history.end())
+                auto history = m_history.get(context());
+                // The history register might have been mutated in the mean time
+                m_current_history = std::min(history.size(), m_current_history);
+                if (m_current_history == history.size())
                    m_prefix = line;
-                auto it = m_history_it;
+                auto index = m_current_history;
                 // search for the previous history entry matching typed prefix
                 do
                 {
-                    --it;
-                    if (prefix_match(*it, m_prefix))
+                    --index;
+                    if (prefix_match(history[index], m_prefix))
                     {
-                        m_history_it = it;
-                        m_line_editor.reset(*it, m_empty_text);
+                        m_current_history = index;
+                        m_line_editor.reset(history[index], m_empty_text);
                         break;
                     }
-                } while (it != history.begin());
+                } while (index != 0);
 
                 clear_completions();
                 m_refresh_completion_pending = true;
@@ -867,16 +879,19 @@ public:
         }
         else if (key == Key::Down or key == ctrl('n')) // next
         {
-            if (m_history_it != history.end())
+            auto history = m_history.get(context());
+            // The history register might have been mutated in the mean time
+            m_current_history = std::min(history.size(), m_current_history);
+            if (m_current_history < history.size())
             {
                 // search for the next history entry matching typed prefix
-                ++m_history_it;
-                while (m_history_it != history.end() and
-                       not prefix_match(*m_history_it, m_prefix))
-                    ++m_history_it;
+                ++m_current_history;
+                while (m_current_history != history.size() and
+                       not prefix_match(history[m_current_history], m_prefix))
+                    ++m_current_history;
 
-                if (m_history_it != history.end())
-                    m_line_editor.reset(*m_history_it, m_empty_text);
+                if (m_current_history != history.size())
+                    m_line_editor.reset(history[m_current_history], m_empty_text);
                 else
                     m_line_editor.reset(m_prefix, m_empty_text);
 
@@ -1001,6 +1016,7 @@ private:
             const String& line = m_line_editor.line();
             m_completions = m_completer(context(), flags, line,
                                         line.byte_count_to(m_line_editor.cursor_pos()));
+            const bool menu = (bool)(m_completions.flags & Completions::Flags::Menu);
             if (context().has_client())
             {
                 if (m_completions.candidates.empty())
@@ -1013,8 +1029,11 @@ private:
                 const auto menu_style = (m_flags & PromptFlags::Search) ? MenuStyle::Search : MenuStyle::Prompt;
                 context().client().menu_show(items, {}, menu_style);
 
+                if (menu)
+                    context().client().menu_select(0);
+
                 auto prefix = line.substr(m_completions.start, m_completions.end - m_completions.start);
-                if (not contains(m_completions.candidates, prefix))
+                if (not menu and not contains(m_completions.candidates, prefix))
                 {
                     m_current_completion = m_completions.candidates.size();
                     m_completions.candidates.push_back(prefix.str());
@@ -1064,8 +1083,8 @@ private:
             context().client().menu_hide();
     }
 
-    PromptCallback m_callback;
-    Completer      m_completer;
+    PromptCallback  m_callback;
+    PromptCompleter m_completer;
     const String   m_prompt;
     Face           m_prompt_face;
     Completions    m_completions;
@@ -1076,33 +1095,28 @@ private:
     LineEditor     m_line_editor;
     bool           m_line_changed = false;
     PromptFlags    m_flags;
+    Register&      m_history;
+    size_t         m_current_history;
     bool           m_auto_complete;
     bool           m_refresh_completion_pending = true;
     Timer          m_idle_timer;
 
-    using History = Vector<String, MemoryDomain::History>;
-    static HashMap<String, History, MemoryDomain::History> ms_history;
-    History::iterator m_history_it;
-
-    void history_push(History& history, StringView entry)
+    void history_push(StringView entry)
     {
         if (entry.empty() or
             (m_flags & PromptFlags::DropHistoryEntriesWithBlankPrefix and
              is_horizontal_blank(entry[0_byte])))
             return;
 
-        history.erase(std::remove(history.begin(), history.end(), entry),
-                      history.end());
-        history.push_back(entry.str());
+        m_history.set(context(), {entry.str()});
     }
 };
-HashMap<String, Prompt::History, MemoryDomain::History> Prompt::ms_history;
 
 class NextKey : public InputMode
 {
 public:
     NextKey(InputHandler& input_handler, KeymapMode keymap_mode, KeyCallback callback)
-        : InputMode(input_handler), m_keymap_mode(keymap_mode), m_callback(std::move(callback)) {}
+        : InputMode(input_handler), m_callback(std::move(callback)), m_keymap_mode(keymap_mode) {}
 
     void on_key(Key key) override
     {
@@ -1132,16 +1146,16 @@ class Insert : public InputMode
 public:
     Insert(InputHandler& input_handler, InsertMode mode, int count)
         : InputMode(input_handler),
-          m_restore_cursor(mode == InsertMode::Append),
           m_edition(context()),
           m_completer(context()),
+          m_restore_cursor(mode == InsertMode::Append),
           m_auto_complete{context().options()["autocomplete"].get<AutoComplete>() & AutoComplete::Insert},
-          m_disable_hooks{context().hooks_disabled(), context().hooks_disabled()},
           m_idle_timer{TimePoint::max(), context().flags() & Context::Flags::Draft ?
                        Timer::Callback{} : [this](Timer&) {
                            m_completer.update(m_auto_complete);
                            context().hooks().run_hook(Hook::InsertIdle, "", context());
-                       }}
+                       }},
+          m_disable_hooks{context().hooks_disabled(), context().hooks_disabled()}
     {
         context().buffer().throw_if_read_only();
 
@@ -1222,6 +1236,8 @@ public:
 
             if (not main_char.empty())
                 context().hooks().run_hook(Hook::InsertDelete, main_char, context());
+
+            context().selections_write_only().update(false);
         }
         else if (key == Key::Delete)
         {
@@ -1568,7 +1584,7 @@ void InputHandler::repeat_last_insert()
     push_mode(new InputModes::Insert(*this, m_last_insert.mode, m_last_insert.count));
     for (auto& key : keys)
     {
-        // refill last_insert,  this is very inefficient, but necesary at the moment
+        // refill last_insert,  this is very inefficient, but necessary at the moment
         // to properly handle insert completion
         m_last_insert.keys.push_back(key);
         current_mode().handle_key(key);
@@ -1577,11 +1593,12 @@ void InputHandler::repeat_last_insert()
 }
 
 void InputHandler::prompt(StringView prompt, String initstr, String emptystr,
-                          Face prompt_face, PromptFlags flags,
-                          Completer completer, PromptCallback callback)
+                          Face prompt_face, PromptFlags flags, char history_register,
+                          PromptCompleter completer, PromptCallback callback)
 {
     push_mode(new InputModes::Prompt(*this, prompt, std::move(initstr), std::move(emptystr),
-                                     prompt_face, flags, std::move(completer), std::move(callback)));
+                                     prompt_face, flags, history_register,
+                                     std::move(completer), std::move(callback)));
 }
 
 void InputHandler::set_prompt_face(Face prompt_face)
