@@ -29,7 +29,7 @@
 #include "clock.hh"
 
 #include <fcntl.h>
-#include <locale>
+#include <locale.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -44,6 +44,13 @@ struct {
     unsigned int version;
     const char* notes;
 } constexpr version_notes[] = { {
+        20190701,
+        "» %file{...} expansions to read files\n"
+        "» echo -to-file <filename> to write to file\n"
+        "» completions option have an on select command instead of a docstring\n"
+        "» Function key syntax do not accept lower case f anymore\n"
+        "» shell quoting of list options is now opt-in with $kak_quoted_...\n"
+    }, {
         20190120,
         "» named capture groups in regex\n"
         "» auto_complete option renamed to autocomplete\n"
@@ -226,6 +233,13 @@ static const EnvVarDesc builtin_env_vars[] = { {
         [](StringView name, const Context& context, Quoting quoting) -> String
         { auto coord = context.selections().main().cursor();
           return to_string(context.buffer()[coord.line].char_count_to(coord.column) + 1); }
+    }, {
+        "cursor_display_column", false,
+        [](StringView name, const Context& context, Quoting quoting) -> String
+        { auto coord = context.selections().main().cursor();
+          return to_string(get_column(context.buffer(),
+                                      context.options()["tabstop"].get<int>(),
+                                      coord) + 1); }
     }, {
         "cursor_byte_offset", false,
         [](StringView name, const Context& context, Quoting quoting) -> String
@@ -268,9 +282,12 @@ void register_registers()
 {
     RegisterManager& register_manager = RegisterManager::instance();
 
-    for (auto c : "abcdefghijklmnopqrstuvwxyz/\"|^@:")
+    for (auto c : StringView{"abcdefghijklmnopqrstuvwxyz\"^@"})
         register_manager.add_register(c, std::make_unique<StaticRegister>());
 
+    for (auto c : StringView{"/|:\\"})
+        register_manager.add_register(c, std::make_unique<HistoryRegister>());
+
     using StringList = Vector<String, MemoryDomain::Registers>;
 
     register_manager.add_register('%', make_dyn_reg(
@@ -399,6 +416,9 @@ void register_options()
     reg.declare_option("autoreload",
                        "autoreload buffer when a filesystem modification is detected",
                        Autoreload::Ask);
+    reg.declare_option("writemethod",
+                       "how to write buffer to files",
+                       WriteMethod::Overwrite);
     reg.declare_option<int, check_timeout>(
         "idle_timeout", "timeout, in milliseconds, before idle hooks are triggered", 50);
     reg.declare_option<int, check_timeout>(
@@ -787,6 +807,7 @@ int run_server(StringView session, StringView server_init,
 
                 if (fork_server_to_background())
                 {
+                    ClientManager::instance().clear(false);
                     String session = server.session();
                     server.close_session(false);
                     throw convert_to_client_mode{ std::move(session), std::move(client_name), std::move(buffer_name), std::move(selections) };
@@ -847,9 +868,11 @@ int run_filter(StringView keystr, ConstArrayView<StringView> files, bool quiet,
         {
             Buffer* buffer = open_file_buffer(file, Buffer::Flags::NoHooks);
             if (not suffix_backup.empty())
-                write_buffer_to_file(*buffer, buffer->name() + suffix_backup);
+                write_buffer_to_file(*buffer, buffer->name() + suffix_backup,
+                                     WriteMethod::Overwrite, WriteFlags::None);
             apply_to_buffer(*buffer);
-            write_buffer_to_file(*buffer, buffer->name());
+            write_buffer_to_file(*buffer, buffer->name(),
+                                 WriteMethod::Overwrite, WriteFlags::None);
             buffer_manager.delete_buffer(*buffer);
         }
         if (not isatty(0))
@@ -1139,3 +1162,17 @@ int main(int argc, char* argv[])
     }
     return 0;
 }
+
+#if defined(__ELF__)
+asm(R"(
+.pushsection ".debug_gdb_scripts", "MS",@progbits,1
+.byte 4
+.ascii "kakoune-inline-gdb.py\n"
+.ascii "import os.path\n"
+.ascii "sys.path.insert(0, os.path.dirname(gdb.current_objfile().filename) + '/../share/kak/gdb/')\n"
+.ascii "import gdb.printing\n"
+.ascii "import kakoune\n"
+.ascii "gdb.printing.register_pretty_printer(gdb.current_objfile(), kakoune.build_pretty_printer())\n\0"
+.popsection
+)");
+#endif
