@@ -250,7 +250,9 @@ default_colors = {
 };
 
 NCursesUI::NCursesUI()
-    : m_stdin_watcher{0, FdEvents::Read,
+    : m_colors{default_colors},
+      m_cursor{CursorMode::Buffer, {}},
+      m_stdin_watcher{0, FdEvents::Read,
                       [this](FDWatcher&, FdEvents, EventMode) {
         if (not m_on_key)
             return;
@@ -258,9 +260,7 @@ NCursesUI::NCursesUI()
         while (auto key = get_next_key())
             m_on_key(*key);
       }},
-      m_assistant(assistant_clippy),
-      m_colors{default_colors},
-      m_cursor{CursorMode::Buffer, {}}
+      m_assistant(assistant_clippy)
 {
     initscr();
     raw();
@@ -286,7 +286,7 @@ NCursesUI::~NCursesUI()
     enable_mouse(false);
     if (can_change_color()) // try to reset palette
     {
-        fputs("\033]104;\007", stdout);
+        fputs("\033]104\007", stdout);
         fflush(stdout);
     }
     endwin();
@@ -1252,7 +1252,7 @@ void NCursesUI::set_ui_options(const Options& options)
 
         if (can_change_color() and m_change_colors != value)
         {
-            fputs("\033]104;\007", stdout); // try to reset palette
+            fputs("\033]104\007", stdout); // try to reset palette
             fflush(stdout);
             m_colorpairs.clear();
             m_colors = default_colors;
