@@ -9,6 +9,7 @@
 #include "commands.hh"
 #include "context.hh"
 #include "diff.hh"
+#include "enum.hh"
 #include "face_registry.hh"
 #include "file.hh"
 #include "flags.hh"
@@ -37,6 +38,14 @@ enum class SelectMode
     Append,
 };
 
+constexpr auto enum_desc(Meta::Type<SelectMode>)
+{
+    return make_array<EnumDesc<SelectMode>, 3>({
+        { SelectMode::Replace, "replace" },
+        { SelectMode::Extend, "extend" },
+        { SelectMode::Append, "append" },
+    });
+}
 void merge_selections(Selection& sel, const Selection& new_sel)
 {
     const bool forward = sel.cursor() >= sel.anchor();
@@ -104,7 +113,7 @@ void select(Context& context, T func)
         }
 
         if (to_remove.size() == selections.size())
-            throw runtime_error{"no selections remaining"};
+            throw no_selections_remaining{};
         for (auto& i : to_remove | reverse())
             selections.remove(i);
     }
@@ -437,7 +446,7 @@ void for_each_codepoint(Context& context, NormalParams)
     selections.insert(strings, InsertMode::Replace);
 }
 
-void command(Context& context, NormalParams params)
+void command(Context& context, EnvVarMap env_vars)
 {
     if (not CommandManager::has_instance())
         throw runtime_error{"commands are not supported"};
@@ -447,11 +456,12 @@ void command(Context& context, NormalParams params)
     context.input_handler().prompt(
         ":", {}, context.main_sel_register_value(':').str(),
         context.faces()["Prompt"], PromptFlags::DropHistoryEntriesWithBlankPrefix,
+        ':',
         [](const Context& context, CompletionFlags flags,
            StringView cmd_line, ByteCount pos) {
                return CommandManager::instance().complete(context, flags, cmd_line, pos);
         },
-        [params](StringView cmdline, PromptEvent event, Context& context) {
+        [env_vars = std::move(env_vars)](StringView cmdline, PromptEvent event, Context& context) {
             if (context.has_client())
             {
                 context.client().info_hide();
@@ -479,46 +489,54 @@ void command(Context& context, NormalParams params)
                 else if (not is_blank(cmdline[0]))
                     RegisterManager::instance()[':'].set(context, cmdline.str());
 
-                EnvVarMap env_vars = {
-                    { "count", to_string(params.count) },
-                    { "register", String{&params.reg, 1} }
-                };
                 CommandManager::instance().execute(
                     cmdline, context, { {}, env_vars });
             }
         });
 }
 
-void apply_diff(Buffer& buffer, BufferCoord pos, StringView before, StringView after)
+void command(Context& context, NormalParams params)
 {
-    // The diff algorithm is O(ND) with N the sum of string len, and D the diff count
-    // do not use it if our data is too big
-    constexpr ByteCount size_limit = 100 * 1024;
-    if (before.length() + after.length() > size_limit)
-    {
-        buffer.erase(pos, buffer.advance(pos, before.length()));
-        buffer.insert(pos, after);
-        return;
-    }
+    EnvVarMap env_vars = {
+        { "count", to_string(params.count) },
+        { "register", String{&params.reg, 1} }
+    };
+    command(context, std::move(env_vars));
+}
 
-    auto diffs = find_diff(before.begin(), (int)before.length(), after.begin(), (int)after.length());
+BufferCoord apply_diff(Buffer& buffer, BufferCoord pos, StringView before, StringView after)
+{
+    const auto lines_before = before | split_after<StringView>('\n') | gather<Vector<StringView>>();
+    const auto lines_after = after | split_after<StringView>('\n') | gather<Vector<StringView>>();
 
+    auto diffs = find_diff(lines_before.begin(), (int)lines_before.size(),
+                           lines_after.begin(), (int)lines_after.size());
+
+    auto byte_count = [](auto&& lines, int first, int count) {
+        return std::accumulate(&lines[first], &lines[first+count], 0_byte,
+                               [](ByteCount l, StringView s) { return l + s.length(); });
+    };
+
+    int posA = 0;
     for (auto& diff : diffs)
     {
         switch (diff.mode)
         {
         case Diff::Keep:
-            pos = buffer.advance(pos, diff.len);
+            pos = buffer.advance(pos, byte_count(lines_before, posA, diff.len));
+            posA += diff.len;
             break;
         case Diff::Add:
-            buffer.insert(pos, after.substr(ByteCount{diff.posB}, ByteCount{diff.len}));
-            pos = buffer.advance(pos, diff.len);
+            pos = buffer.insert(pos, {lines_after[diff.posB].begin(),
+                                      lines_after[diff.posB + diff.len - 1].end()});
             break;
         case Diff::Remove:
-            buffer.erase(pos, buffer.advance(pos, diff.len));
+            pos = buffer.erase(pos, buffer.advance(pos, byte_count(lines_before, posA, diff.len)));
+            posA += diff.len;
             break;
         }
     }
+    return pos;
 }
 
 template<bool replace>
@@ -527,7 +545,7 @@ void pipe(Context& context, NormalParams)
     const char* prompt = replace ? "pipe:" : "pipe-to:";
     context.input_handler().prompt(
         prompt, {}, context.main_sel_register_value("|").str(), context.faces()["Prompt"],
-        PromptFlags::DropHistoryEntriesWithBlankPrefix,
+        PromptFlags::DropHistoryEntriesWithBlankPrefix, '|',
         shell_complete,
         [](StringView cmdline, PromptEvent event, Context& context)
         {
@@ -544,19 +562,14 @@ void pipe(Context& context, NormalParams)
 
             Buffer& buffer = context.buffer();
             SelectionList selections = context.selections();
-            auto restore_sels = on_scope_end([&, old_main = selections.main_index()] {
-                selections.set_main_index(old_main);
-                context.selections() = std::move(selections);
-            });
             if (replace)
             {
                 ScopedEdition edition(context);
                 ForwardChangesTracker changes_tracker;
                 size_t timestamp = buffer.timestamp();
-                for (int i = 0; i < selections.size(); ++i)
+                Vector<Selection> new_sels;
+                for (auto& sel : selections)
                 {
-                    selections.set_main_index(i);
-                    auto& sel = selections.main();
                     const auto beg = changes_tracker.get_new_coord_tolerant(sel.min());
                     const auto end = changes_tracker.get_new_coord_tolerant(sel.max());
 
@@ -566,7 +579,7 @@ void pipe(Context& context, NormalParams)
                         in += '\n';
 
                     // Needed in case we read selections inside the cmdline
-                    context.selections_write_only() = selections;
+                    context.selections_write_only().set({keep_direction(Selection{beg, end}, sel)}, 0);
 
                     String out = ShellManager::instance().eval(
                         cmdline, context, in,
@@ -578,13 +591,23 @@ void pipe(Context& context, NormalParams)
                         if (not out.empty() and out.back() == '\n')
                             out.resize(out.length()-1, 0);
                     }
-                    apply_diff(buffer, beg, in, out);
+                    auto new_end = apply_diff(buffer, beg, in, out);
+                    if (new_end != beg)
+                        new_sels.push_back(keep_direction({beg, buffer.char_prev(new_end), std::move(sel.captures())}, sel));
+                    else
+                    {
+                        if (new_end != BufferCoord{})
+                            new_end = buffer.char_prev(new_end);
+                        new_sels.push_back({new_end, new_end, std::move(sel.captures())});
+                    }
 
                     changes_tracker.update(buffer, timestamp);
                 }
+                context.selections_write_only().set(std::move(new_sels), selections.main_index());
             }
             else
             {
+                const auto old_main = selections.main_index();
                 for (int i = 0; i < selections.size(); ++i)
                 {
                     selections.set_main_index(i);
@@ -592,6 +615,7 @@ void pipe(Context& context, NormalParams)
                                                   content(buffer, selections.main()),
                                                   ShellManager::Flags::None);
                 }
+                selections.set_main_index(old_main);
             }
         });
 }
@@ -602,7 +626,7 @@ void insert_output(Context& context, NormalParams)
     const char* prompt = mode == InsertMode::Insert ? "insert-output:" : "append-output:";
     context.input_handler().prompt(
         prompt, {}, context.main_sel_register_value("|").str(), context.faces()["Prompt"],
-        PromptFlags::DropHistoryEntriesWithBlankPrefix,
+        PromptFlags::DropHistoryEntriesWithBlankPrefix, '|',
         shell_complete,
         [](StringView cmdline, PromptEvent event, Context& context)
         {
@@ -723,20 +747,22 @@ void paste_all(Context& context, NormalParams params)
         selections = std::move(result);
 }
 
-constexpr RegexCompileFlags direction_flags(MatchDirection direction)
+constexpr RegexCompileFlags direction_flags(RegexMode mode)
 {
-    return (direction == MatchDirection::Forward) ?
+    return (mode & RegexMode::Forward) ?
         RegexCompileFlags::None : RegexCompileFlags::Backward | RegexCompileFlags::NoForward;
 }
 
-template<MatchDirection direction = MatchDirection::Forward, typename T>
-void regex_prompt(Context& context, String prompt, String default_regex, T func)
+template<RegexMode mode = RegexMode::Forward, typename T>
+void regex_prompt(Context& context, String prompt, char reg, T func)
 {
+    static_assert(is_direction(mode));
     DisplayCoord position = context.has_window() ? context.window().position() : DisplayCoord{};
     SelectionList selections = context.selections();
+    auto default_regex = RegisterManager::instance()[reg].get_main(context, context.selections().main_index());
     context.input_handler().prompt(
         std::move(prompt), {}, default_regex, context.faces()["Prompt"],
-        PromptFlags::Search,
+        PromptFlags::Search, reg,
         [](const Context& context, CompletionFlags, StringView regex, ByteCount pos) -> Completions {
             auto current_word = [](StringView s) {
                 auto it = s.end();
@@ -761,7 +787,7 @@ void regex_prompt(Context& context, String prompt, String default_regex, T func)
                        [&](auto&& m) { candidates.push_back(m.candidate().str()); return true; });
             return {(int)(word.begin() - regex.begin()), pos,  std::move(candidates) };
         },
-        [=](StringView str, PromptEvent event, Context& context) mutable {
+        [=, func=T(std::move(func))](StringView str, PromptEvent event, Context& context) mutable {
             try
             {
                 if (event != PromptEvent::Change and context.has_client())
@@ -785,7 +811,7 @@ void regex_prompt(Context& context, String prompt, String default_regex, T func)
                     context.push_jump();
 
                 if (not str.empty() or event == PromptEvent::Validate)
-                    func(Regex{str.empty() ? default_regex : str, direction_flags(direction)}, event, context);
+                    func(Regex{str.empty() ? default_regex : str, direction_flags(mode)}, event, context);
             }
             catch (regex_error& err)
             {
@@ -805,19 +831,19 @@ void regex_prompt(Context& context, String prompt, String default_regex, T func)
         });
 }
 
-template<MatchDirection direction>
+template<RegexMode mode>
 void select_next_matches(Context& context, const Regex& regex, int count)
 {
      auto& selections = context.selections();
      do {
          bool wrapped = false;
          for (auto& sel : selections)
-             sel = keep_direction(find_next_match<direction>(context, sel, regex, wrapped), sel);
+             sel = keep_direction(find_next_match<mode>(context, sel, regex, wrapped), sel);
          selections.sort_and_merge_overlapping();
      } while (--count > 0);
 }
 
-template<MatchDirection direction>
+template<RegexMode mode>
 void extend_to_next_matches(Context& context, const Regex& regex, int count)
 {
      Vector<Selection> new_sels;
@@ -827,7 +853,7 @@ void extend_to_next_matches(Context& context, const Regex& regex, int count)
          size_t main_index = selections.main_index();
          for (auto& sel : selections)
          {
-             auto new_sel = find_next_match<direction>(context, sel, regex, wrapped);
+             auto new_sel = find_next_match<mode>(context, sel, regex, wrapped);
              if (not wrapped)
              {
                  new_sels.push_back(sel);
@@ -844,48 +870,43 @@ void extend_to_next_matches(Context& context, const Regex& regex, int count)
      } while (--count > 0);
 }
 
-template<SelectMode mode, MatchDirection direction>
+template<SelectMode mode, RegexMode regex_mode>
 void search(Context& context, NormalParams params)
 {
+    static_assert(is_direction(regex_mode));
     constexpr StringView prompt = mode == SelectMode::Extend ?
-        (direction == MatchDirection::Forward ? "search (extend):" : "reverse search (extend):")
-      : (direction == MatchDirection::Forward ? "search:"          : "reverse search:");
+        (regex_mode & RegexMode::Forward ? "search (extend):" : "reverse search (extend):")
+      : (regex_mode & RegexMode::Forward ? "search:"          : "reverse search:");
 
     const char reg = to_lower(params.reg ? params.reg : '/');
     const int count = params.count;
 
-    auto reg_content = RegisterManager::instance()[reg].get(context);
-    Vector<String> saved_reg{reg_content.begin(), reg_content.end()};
-    const int main_index = std::min(context.selections().main_index(), saved_reg.size()-1);
-
-    regex_prompt<direction>(context, prompt.str(), saved_reg[main_index],
-                 [reg, count, saved_reg]
+    regex_prompt<regex_mode>(context, prompt.str(), reg,
+                 [reg, count, saved_reg = RegisterManager::instance()[reg].save(context)]
                  (const Regex& regex, PromptEvent event, Context& context) {
+                     RegisterManager::instance()[reg].restore(context, saved_reg);
                      if (event == PromptEvent::Abort)
-                     {
-                         RegisterManager::instance()[reg].set(context, saved_reg);
                          return;
-                     }
                      RegisterManager::instance()[reg].set(context, regex.str());
 
                      if (regex.empty() or regex.str().empty())
                          return;
 
                      if (mode == SelectMode::Extend)
-                         extend_to_next_matches<direction>(context, regex, count);
+                         extend_to_next_matches<regex_mode>(context, regex, count);
                      else
-                         select_next_matches<direction>(context, regex, count);
+                         select_next_matches<regex_mode>(context, regex, count);
                  });
 }
 
-template<SelectMode mode, MatchDirection direction>
+template<SelectMode mode, RegexMode regex_mode>
 void search_next(Context& context, NormalParams params)
 {
     const char reg = to_lower(params.reg ? params.reg : '/');
-    StringView str = context.main_sel_register_value(reg);
+    StringView str = RegisterManager::instance()[reg].get(context).back();
     if (not str.empty())
     {
-        Regex regex{str, direction_flags(direction)};
+        Regex regex{str, direction_flags(regex_mode)};
         auto& selections = context.selections();
         bool main_wrapped = false;
         do {
@@ -893,12 +914,12 @@ void search_next(Context& context, NormalParams params)
             if (mode == SelectMode::Replace)
             {
                 auto& sel = selections.main();
-                sel = keep_direction(find_next_match<direction>(context, sel, regex, wrapped), sel);
+                sel = keep_direction(find_next_match<regex_mode>(context, sel, regex, wrapped), sel);
             }
             else if (mode == SelectMode::Append)
             {
                 auto sel = keep_direction(
-                    find_next_match<direction>(context, selections.main(), regex, wrapped),
+                    find_next_match<regex_mode>(context, selections.main(), regex, wrapped),
                     selections.main());
                 selections.push_back(std::move(sel));
                 selections.set_main_index(selections.size() - 1);
@@ -917,25 +938,21 @@ void search_next(Context& context, NormalParams params)
 template<bool smart>
 void use_selection_as_search_pattern(Context& context, NormalParams params)
 {
-    Vector<String> patterns;
-    auto& sels = context.selections();
     const auto& buffer = context.buffer();
-    for (auto& sel : sels)
-    {
-        const auto beg = sel.min(), end = buffer.char_next(sel.max());
-        patterns.push_back(format("{}{}{}",
-                                  smart and is_bow(buffer, beg) ? "\\b" : "",
-                                  escape(buffer.string(beg, end), "^$\\.*+?()[]{}|", '\\'),
-                                  smart and is_eow(buffer, end) ? "\\b" : ""));
-    }
+    auto& sel = context.selections().main();
+    const auto beg = sel.min(), end = buffer.char_next(sel.max());
+    String pattern = format("{}{}{}",
+                            smart and is_bow(buffer, beg) ? "\\b" : "",
+                            escape(buffer.string(beg, end), "^$\\.*+?()[]{}|", '\\'),
+                            smart and is_eow(buffer, end) ? "\\b" : "");
 
     const char reg = to_lower(params.reg ? params.reg : '/');
 
     context.print_status({
-        format("register '{}' set to '{}'", reg, fix_atom_text(patterns[sels.main_index()])),
+        format("register '{}' set to '{}'", reg, fix_atom_text(pattern)),
         context.faces()["Information"] });
 
-    RegisterManager::instance()[reg].set(context, patterns);
+    RegisterManager::instance()[reg].set(context, {pattern});
 
     // Hack, as Window do not take register state into account
     if (context.has_window())
@@ -948,22 +965,19 @@ void select_regex(Context& context, NormalParams params)
     const int capture = params.count;
     auto prompt = capture ? format("select (capture {}):", capture) :  "select:"_str;
 
-    auto reg_content = RegisterManager::instance()[reg].get(context);
-    Vector<String> saved_reg{reg_content.begin(), reg_content.end()};
-    const int main_index = std::min(context.selections().main_index(), saved_reg.size()-1);
-
-    regex_prompt(context, std::move(prompt), saved_reg[main_index],
-                 [reg, capture, saved_reg](Regex ex, PromptEvent event, Context& context) {
-         if (event == PromptEvent::Abort)
-         {
-             RegisterManager::instance()[reg].set(context, saved_reg);
-             return;
-         }
-
-        RegisterManager::instance()[reg].set(context, ex.str());
+    regex_prompt(context, std::move(prompt), reg,
+                 [reg, capture, saved_reg = RegisterManager::instance()[reg].save(context)]
+                 (Regex ex, PromptEvent event, Context& context) {
+        RegisterManager::instance()[reg].restore(context, saved_reg);
+        if (event == PromptEvent::Abort)
+            return;
+        if (not context.history_disabled())
+            RegisterManager::instance()[reg].set(context, ex.str());
 
+        auto& selections = context.selections();
+        auto& buffer = selections.buffer();
         if (not ex.empty() and not ex.str().empty())
-            select_all_matches(context.selections(), ex, capture);
+            selections = SelectionList{buffer, select_matches(buffer, selections, ex, capture)};
     });
 }
 
@@ -973,22 +987,19 @@ void split_regex(Context& context, NormalParams params)
     const int capture = params.count;
     auto prompt = capture ? format("split (on capture {}):", (int)capture) :  "split:"_str;
 
-    auto reg_content = RegisterManager::instance()[reg].get(context);
-    Vector<String> saved_reg{reg_content.begin(), reg_content.end()};
-    const int main_index = std::min(context.selections().main_index(), saved_reg.size()-1);
-
-    regex_prompt(context, std::move(prompt), saved_reg[main_index],
-                 [reg, capture, saved_reg](Regex ex, PromptEvent event, Context& context) {
-         if (event == PromptEvent::Abort)
-         {
-             RegisterManager::instance()[reg].set(context, saved_reg);
-             return;
-         }
-
-        RegisterManager::instance()[reg].set(context, ex.str());
+    regex_prompt(context, std::move(prompt), reg,
+                 [reg, capture, saved_reg = RegisterManager::instance()[reg].save(context)]
+                 (Regex ex, PromptEvent event, Context& context) {
+        RegisterManager::instance()[reg].restore(context, saved_reg);
+        if (event == PromptEvent::Abort)
+            return;
+        if (not context.history_disabled())
+            RegisterManager::instance()[reg].set(context, ex.str());
 
+        auto& selections = context.selections();
+        auto& buffer = selections.buffer();
         if (not ex.empty() and not ex.str().empty())
-            split_selections(context.selections(), ex, capture);
+            selections = SelectionList{buffer, split_on_matches(buffer, selections, ex, capture)};
     });
 }
 
@@ -1073,18 +1084,13 @@ void keep(Context& context, NormalParams params)
     constexpr StringView prompt = matching ? "keep matching:" : "keep not matching:";
 
     const char reg = to_lower(params.reg ? params.reg : '/');
-    auto saved_reg = RegisterManager::instance()[reg].get(context) | gather<Vector<String>>();
-    const int main_index = std::min(context.selections().main_index(), saved_reg.size()-1);
 
-    regex_prompt(context, prompt.str(), saved_reg[main_index],
-                 [saved_reg, reg]
+    regex_prompt(context, prompt.str(), reg,
+                 [reg, saved_reg = RegisterManager::instance()[reg].save(context)]
                  (const Regex& regex, PromptEvent event, Context& context) {
-
+        RegisterManager::instance()[reg].restore(context, saved_reg);
         if (event == PromptEvent::Abort)
-        {
-            RegisterManager::instance()[reg].set(context, saved_reg);
             return;
-        }
         if (not context.history_disabled())
             RegisterManager::instance()[reg].set(context, regex.str());
 
@@ -1101,13 +1107,12 @@ void keep(Context& context, NormalParams params)
             // give more intuitive behaviours in keep use cases.
             const auto flags = match_flags(is_bol(begin.coord()), false,
                                            is_bow(buffer, begin.coord()),
-                                           is_eow(buffer, end.coord())) |
-                               RegexExecFlags::AnyMatch;
+                                           is_eow(buffer, end.coord()));
             if (regex_search(begin, end, begin, end, regex, flags) == matching)
                 keep.push_back(sel);
         }
         if (keep.empty())
-            throw runtime_error("no selections remaining");
+            throw no_selections_remaining{};
         context.selections_write_only() = std::move(keep);
     });
 }
@@ -1116,7 +1121,7 @@ void keep_pipe(Context& context, NormalParams)
 {
     context.input_handler().prompt(
         "keep pipe:", {}, {}, context.faces()["Prompt"],
-        PromptFlags::DropHistoryEntriesWithBlankPrefix, shell_complete,
+        PromptFlags::DropHistoryEntriesWithBlankPrefix, '|', shell_complete,
         [](StringView cmdline, PromptEvent event, Context& context) {
             if (event != PromptEvent::Validate)
                 return;
@@ -1140,7 +1145,7 @@ void keep_pipe(Context& context, NormalParams)
                 }
             }
             if (keep.empty())
-                throw runtime_error("no selections remaining");
+                throw no_selections_remaining{};
             if (new_main == -1)
                 new_main = keep.size() - 1;
             context.selections_write_only().set(std::move(keep), new_main);
@@ -1238,12 +1243,12 @@ void select_object(Context& context, NormalParams params)
                       whole ? "" : (flags & ObjectFlags::ToBegin ? " begin" : " end"));
     };
 
-    const int count = params.count <= 0 ? 0 : params.count - 1;
     on_next_key_with_autoinfo(context, KeymapMode::Object,
-                             [count](Key key, Context& context) {
+                             [params](Key key, Context& context) {
         if (key == Key::Escape)
             return;
 
+        const int count = params.count <= 0 ? 0 : params.count - 1;
         static constexpr struct ObjectType
         {
             Key key;
@@ -1273,7 +1278,7 @@ void select_object(Context& context, NormalParams params)
 
             context.input_handler().prompt(
                 "object desc:", {}, {}, context.faces()["Prompt"],
-                PromptFlags::None, complete_nothing,
+                PromptFlags::None, '_', complete_nothing,
                 [count,info](StringView cmdline, PromptEvent event, Context& context) {
                     if (event != PromptEvent::Change)
                         hide_auto_info_ifn(context, info);
@@ -1297,6 +1302,18 @@ void select_object(Context& context, NormalParams params)
             return;
         }
 
+        if (key == alt(';'))
+        {
+            EnvVarMap env_vars = {
+                { "count", to_string(params.count) },
+                { "register", String{&params.reg, 1} },
+                { "select_mode", option_to_string(mode, Quoting::Raw) },
+                { "object_flags", option_to_string(flags, Quoting::Raw) }
+            };
+            command(context, std::move(env_vars));
+            return;
+        }
+
         static constexpr struct SurroundingPair
         {
             char opening;
@@ -1351,7 +1368,8 @@ void select_object(Context& context, NormalParams params)
          {{'i'},         "indent"},
          {{'u'},         "argument"},
          {{'n'},         "number"},
-         {{'c'},         "custom object desc"}}));
+         {{'c'},         "custom object desc"},
+         {{alt(';')},    "run command in object context"}}));
 }
 
 enum Direction { Backward = -1, Forward = 1 };
@@ -1734,7 +1752,7 @@ void trim_selections(Context& context, NormalParams)
     }
 
     if (to_remove.size() == selections.size())
-        throw runtime_error{"no selections remaining"};
+        throw no_selections_remaining{};
     for (auto& i : to_remove | reverse())
         selections.remove(i);
 }
@@ -2047,7 +2065,8 @@ void move_cursor(Context& context, NormalParams params)
 
 void select_whole_buffer(Context& context, NormalParams)
 {
-    select_buffer(context.selections());
+    auto& buffer = context.buffer();
+    context.selections_write_only() = SelectionList{buffer, {{0,0}, {buffer.back_coord(), max_column}}};
 }
 
 void keep_selection(Context& context, NormalParams p)
@@ -2117,10 +2136,39 @@ void force_redraw(Context& context, NormalParams)
     }
 }
 
-template<typename T, MemoryDomain domain>
-using KeymapBackend = Vector<T, domain>;
+constexpr size_t keymap_max_size = 512;
 
-static const HashMap<Key, NormalCmd, MemoryDomain::Undefined, KeymapBackend> keymap = {
+template<typename T, MemoryDomain domain>
+struct KeymapBackend : private Array<T, keymap_max_size>
+{
+    using iterator = const T*;
+    using const_iterator = const T*;
+
+    constexpr KeymapBackend() = default;
+    constexpr KeymapBackend(std::initializer_list<T> items)
+    {
+        resize(items.size());
+        for (size_t i = 0; i < items.size(); ++i)
+            this->m_data[i] = *(items.begin() + i);
+    }
+
+    constexpr void resize(size_t s)
+    {
+        if (s > keymap_max_size)
+            throw runtime_error{"cannot resize"};
+        m_size = s;
+    }
+    constexpr size_t size() const { return m_size; }
+
+    using KeymapBackend::Array::begin;
+    using KeymapBackend::Array::end;
+    using KeymapBackend::Array::operator[];
+
+private:
+    size_t m_size = 0;
+};
+
+static constexpr HashMap<Key, NormalCmd, MemoryDomain::Undefined, KeymapBackend> keymap = {
     { {'h'}, {"move left", move_cursor<CharCount, Backward>} },
     { {'j'}, {"move down", move_cursor<LineCount, Forward>} },
     { {'k'}, {"move up",  move_cursor<LineCount, Backward>} },
@@ -2235,14 +2283,14 @@ static const HashMap<Key, NormalCmd, MemoryDomain::Undefined, KeymapBackend> key
     { {'M'}, {"extend to matching character", select<SelectMode::Extend, select_matching<true>>} },
     { {alt('M')}, {"backward extend to matching character", select<SelectMode::Extend, select_matching<false>>} },
 
-    { {'/'}, {"select next given regex match", search<SelectMode::Replace, MatchDirection::Forward>} },
-    { {'?'}, {"extend with next given regex match", search<SelectMode::Extend, MatchDirection::Forward>} },
-    { {alt('/')}, {"select previous given regex match", search<SelectMode::Replace, MatchDirection::Backward>} },
-    { {alt('?')}, {"extend with previous given regex match", search<SelectMode::Extend, MatchDirection::Backward>} },
-    { {'n'}, {"select next current search pattern match", search_next<SelectMode::Replace, MatchDirection::Forward>} },
-    { {'N'}, {"extend with next current search pattern match", search_next<SelectMode::Append, MatchDirection::Forward>} },
-    { {alt('n')}, {"select previous current search pattern match", search_next<SelectMode::Replace, MatchDirection::Backward>} },
-    { {alt('N')}, {"extend with previous current search pattern match", search_next<SelectMode::Append, MatchDirection::Backward>} },
+    { {'/'}, {"select next given regex match", search<SelectMode::Replace, RegexMode::Forward>} },
+    { {'?'}, {"extend with next given regex match", search<SelectMode::Extend, RegexMode::Forward>} },
+    { {alt('/')}, {"select previous given regex match", search<SelectMode::Replace, RegexMode::Backward>} },
+    { {alt('?')}, {"extend with previous given regex match", search<SelectMode::Extend, RegexMode::Backward>} },
+    { {'n'}, {"select next current search pattern match", search_next<SelectMode::Replace, RegexMode::Forward>} },
+    { {'N'}, {"extend with next current search pattern match", search_next<SelectMode::Append, RegexMode::Forward>} },
+    { {alt('n')}, {"select previous current search pattern match", search_next<SelectMode::Replace, RegexMode::Backward>} },
+    { {alt('N')}, {"extend with previous current search pattern match", search_next<SelectMode::Append, RegexMode::Backward>} },
     { {'*'}, {"set search pattern to main selection content", use_selection_as_search_pattern<true>} },
     { {alt('*')}, {"set search pattern to main selection content, do not detect words", use_selection_as_search_pattern<false>} },
 
