@@ -91,46 +91,53 @@ struct ParsedRegex
 
 namespace
 {
-template<MatchDirection = MatchDirection::Forward>
-struct ForEachChild
+
+template<RegexMode mode = RegexMode::Forward>
+struct Children
 {
-    template<typename Func>
-    static bool apply(const ParsedRegex& parsed_regex, ParsedRegex::NodeIndex index, Func&& func)
+    static_assert(has_direction(mode));
+    using Index = ParsedRegex::NodeIndex;
+    struct Sentinel {};
+    struct Iterator
     {
-        const auto end = parsed_regex.nodes[index].children_end;
-        for (auto child = index+1; child != end;
-             child = parsed_regex.nodes[child].children_end)
+        static constexpr bool forward = mode & RegexMode::Forward;
+        Iterator(ArrayView<const ParsedRegex::Node> nodes, Index index)
+          : m_nodes{nodes},
+            m_pos(forward ? index+1 : find_prev(index, nodes[index].children_end)),
+            m_end(forward ? nodes[index].children_end : index)
+        {}
+
+        Iterator& operator++()
         {
-            if (func(child) == false)
-                return false;
+            m_pos = forward ? m_nodes[m_pos].children_end : find_prev(m_end, m_pos);
+            return *this;
         }
-        return true;
-    }
+
+        Index operator*() const { return m_pos; }
+        bool operator!=(Sentinel) const { return m_pos != m_end; }
+
+        Index find_prev(Index parent, Index pos) const
+        {
+            Index child = parent+1;
+            if (child == pos)
+                return parent;
+            while (m_nodes[child].children_end != pos)
+                child = m_nodes[child].children_end;
+            return child;
+        }
+
+        ArrayView<const ParsedRegex::Node> m_nodes;
+        Index m_pos;
+        Index m_end;
+    };
+
+    Iterator begin() const { return {m_parsed_regex.nodes, m_index}; }
+    Sentinel end() const { return {}; }
+
+    const ParsedRegex& m_parsed_regex;
+    const Index m_index;
 };
 
-template<>
-struct ForEachChild<MatchDirection::Backward>
-{
-    template<typename Func>
-    static bool apply(const ParsedRegex& parsed_regex, ParsedRegex::NodeIndex index, Func&& func)
-    {
-        auto find_last_child = [&](ParsedRegex::NodeIndex begin, ParsedRegex::NodeIndex end) {
-            while (parsed_regex.nodes[begin].children_end != end)
-                begin = parsed_regex.nodes[begin].children_end;
-            return begin;
-        };
-        const auto first_child = index+1;
-        auto end = parsed_regex.nodes[index].children_end;
-        while (end != first_child)
-        {
-            auto child = find_last_child(first_child, end);
-            if (func(child) == false)
-                return false;
-            end = child;
-        }
-        return true;
-    }
-};
 }
 
 // Recursive descent parser based on naming used in the ECMAScript
@@ -615,7 +622,8 @@ private:
     void validate_lookaround(NodeIndex index)
     {
         using Lookaround = CompiledRegex::Lookaround;
-        ForEachChild<>::apply(m_parsed_regex, index, [this](NodeIndex child_index) {
+        for (auto child_index : Children<>{m_parsed_regex, index})
+        {
             auto& child = get_node(child_index);
             if (child.op != ParsedRegex::Literal and child.op != ParsedRegex::Class and
                 child.op != ParsedRegex::CharacterType and child.op != ParsedRegex::AnyChar and
@@ -627,8 +635,7 @@ private:
                 parse_error("Lookaround does not support literals codepoint between 0xF0000 and 0xFFFFD");
             if (child.quantifier.type != ParsedRegex::Quantifier::One)
                 parse_error("Quantifiers cannot be used in lookarounds");
-            return true;
-        });
+        }
     }
 
     ParsedRegex m_parsed_regex;
@@ -665,28 +672,26 @@ constexpr RegexParser::ControlEscape RegexParser::control_escapes[];
 struct RegexCompiler
 {
     RegexCompiler(ParsedRegex&& parsed_regex, RegexCompileFlags flags)
-        : m_parsed_regex{parsed_regex}, m_flags(flags)
+        : m_flags(flags), m_parsed_regex{parsed_regex}
     {
         kak_assert(not (flags & RegexCompileFlags::NoForward) or flags & RegexCompileFlags::Backward);
         // Approximation of the number of instructions generated
-        m_program.instructions.reserve((CompiledRegex::search_prefix_size + parsed_regex.nodes.size() + 1)
+        m_program.instructions.reserve((parsed_regex.nodes.size() + 1)
                                        * (((flags & RegexCompileFlags::Backward) and
                                            not (flags & RegexCompileFlags::NoForward)) ? 2 : 1));
 
         if (not (flags & RegexCompileFlags::NoForward))
         {
-            m_program.forward_start_desc = compute_start_desc<MatchDirection::Forward>();
-            write_search_prefix();
-            compile_node<MatchDirection::Forward>(0);
+            m_program.forward_start_desc = compute_start_desc<RegexMode::Forward>();
+            compile_node<RegexMode::Forward>(0);
             push_inst(CompiledRegex::Match);
         }
 
         if (flags & RegexCompileFlags::Backward)
         {
             m_program.first_backward_inst = m_program.instructions.size();
-            m_program.backward_start_desc = compute_start_desc<MatchDirection::Backward>();
-            write_search_prefix();
-            compile_node<MatchDirection::Backward>(0);
+            m_program.backward_start_desc = compute_start_desc<RegexMode::Backward>();
+            compile_node<RegexMode::Backward>(0);
             push_inst(CompiledRegex::Match);
         }
         else
@@ -701,7 +706,7 @@ struct RegexCompiler
 
 private:
 
-    template<MatchDirection direction>
+    template<RegexMode direction>
     uint32_t compile_node_inner(ParsedRegex::NodeIndex index)
     {
         auto& node = get_node(index);
@@ -711,7 +716,7 @@ private:
 
         const bool save = (node.op == ParsedRegex::Alternation or node.op == ParsedRegex::Sequence) and
                           (node.value == 0 or (node.value != -1 and not (m_flags & RegexCompileFlags::NoSubs)));
-        constexpr bool forward = direction == MatchDirection::Forward;
+        constexpr bool forward = direction == RegexMode::Forward;
         if (save)
             push_inst(CompiledRegex::Save, node.value * 2 + (forward ? 0 : 1));
 
@@ -738,22 +743,22 @@ private:
                 break;
             case ParsedRegex::Sequence:
             {
-                ForEachChild<direction>::apply(m_parsed_regex, index, [this](ParsedRegex::NodeIndex child) {
-                    compile_node<direction>(child); return true;
-                });
+                for (auto child : Children<direction>{m_parsed_regex, index})
+                    compile_node<direction>(child);
                 break;
             }
             case ParsedRegex::Alternation:
             {
                 auto split_pos = m_program.instructions.size();
-                ForEachChild<>::apply(m_parsed_regex, index, [this, index](ParsedRegex::NodeIndex child) {
+                for (auto child : Children<>{m_parsed_regex, index})
+                {
                     if (child != index+1)
                         push_inst(CompiledRegex::Split_PrioritizeParent);
-                    return true;
-                });
+                }
 
-                ForEachChild<>::apply(m_parsed_regex, index,
-                                      [&, end = node.children_end](ParsedRegex::NodeIndex  child) {
+                const auto end = node.children_end;
+                for (auto child : Children<>{m_parsed_regex, index})
+                {
                     auto node = compile_node<direction>(child);
                     if (child != index+1)
                         m_program.instructions[split_pos++].param = node;
@@ -762,29 +767,28 @@ private:
                         auto jump = push_inst(CompiledRegex::Jump);
                         goto_inner_end_offsets.push_back(jump);
                     }
-                    return true;
-                });
+                }
                 break;
             }
             case ParsedRegex::LookAhead:
                 push_inst(ignore_case ? CompiledRegex::LookAhead_IgnoreCase
                                       : CompiledRegex::LookAhead,
-                          push_lookaround<MatchDirection::Forward>(index, ignore_case));
+                          push_lookaround<RegexMode::Forward>(index, ignore_case));
                 break;
             case ParsedRegex::NegativeLookAhead:
                 push_inst(ignore_case ? CompiledRegex::NegativeLookAhead_IgnoreCase
                                       : CompiledRegex::NegativeLookAhead,
-                          push_lookaround<MatchDirection::Forward>(index, ignore_case));
+                          push_lookaround<RegexMode::Forward>(index, ignore_case));
                 break;
             case ParsedRegex::LookBehind:
                 push_inst(ignore_case ? CompiledRegex::LookBehind_IgnoreCase
                                       : CompiledRegex::LookBehind,
-                          push_lookaround<MatchDirection::Backward>(index, ignore_case));
+                          push_lookaround<RegexMode::Backward>(index, ignore_case));
                 break;
             case ParsedRegex::NegativeLookBehind:
                 push_inst(ignore_case ? CompiledRegex::NegativeLookBehind_IgnoreCase
                                       : CompiledRegex::NegativeLookBehind,
-                          push_lookaround<MatchDirection::Backward>(index, ignore_case));
+                          push_lookaround<RegexMode::Backward>(index, ignore_case));
                 break;
             case ParsedRegex::LineStart:
                 push_inst(CompiledRegex::LineStart);
@@ -818,7 +822,7 @@ private:
         return start_pos;
     }
 
-    template<MatchDirection direction>
+    template<RegexMode direction>
     uint32_t compile_node(ParsedRegex::NodeIndex index)
     {
         auto& node = get_node(index);
@@ -860,16 +864,6 @@ private:
         return start_pos;
     }
 
-    // Add a sequence of instructions that enable searching for a match instead of checking for it
-    void write_search_prefix()
-    {
-        const uint32_t first_inst = m_program.instructions.size();
-        push_inst(CompiledRegex::Split_PrioritizeChild, first_inst + CompiledRegex::search_prefix_size);
-        push_inst(CompiledRegex::FindNextStart);
-        push_inst(CompiledRegex::Split_PrioritizeParent, first_inst + 1);
-        kak_assert(m_program.instructions.size() == first_inst + CompiledRegex::search_prefix_size);
-    }
-
     uint32_t push_inst(CompiledRegex::Op op, uint32_t param = 0)
     {
         constexpr auto max_instructions = std::numeric_limits<int16_t>::max();
@@ -880,32 +874,29 @@ private:
         return res;
     }
 
-    template<MatchDirection direction>
+    template<RegexMode direction>
     uint32_t push_lookaround(ParsedRegex::NodeIndex index, bool ignore_case)
     {
         using Lookaround = CompiledRegex::Lookaround;
 
         const uint32_t res = m_program.lookarounds.size();
-        auto write_matcher = [this, ignore_case](ParsedRegex::NodeIndex  child) {
-                auto& character = get_node(child);
-                if (character.op == ParsedRegex::Literal)
-                    m_program.lookarounds.push_back(
-                        static_cast<Lookaround>(ignore_case ? to_lower(character.value) : character.value));
-                else if (character.op == ParsedRegex::AnyChar)
-                    m_program.lookarounds.push_back(Lookaround::AnyChar);
-                else if (character.op == ParsedRegex::AnyCharExceptNewLine)
-                    m_program.lookarounds.push_back(Lookaround::AnyCharExceptNewLine);
-                else if (character.op == ParsedRegex::Class)
-                    m_program.lookarounds.push_back(static_cast<Lookaround>(to_underlying(Lookaround::CharacterClass) + character.value));
-                else if (character.op == ParsedRegex::CharacterType)
-                    m_program.lookarounds.push_back(static_cast<Lookaround>(to_underlying(Lookaround::CharacterType) | character.value));
-                else
-                    kak_assert(false);
-                return true;
-        };
-
-        ForEachChild<direction>::apply(m_parsed_regex, index, write_matcher);
-
+        for (auto child : Children<direction>{m_parsed_regex, index})
+        {
+            auto& character = get_node(child);
+            if (character.op == ParsedRegex::Literal)
+                m_program.lookarounds.push_back(
+                    static_cast<Lookaround>(ignore_case ? to_lower(character.value) : character.value));
+            else if (character.op == ParsedRegex::AnyChar)
+                m_program.lookarounds.push_back(Lookaround::AnyChar);
+            else if (character.op == ParsedRegex::AnyCharExceptNewLine)
+                m_program.lookarounds.push_back(Lookaround::AnyCharExceptNewLine);
+            else if (character.op == ParsedRegex::Class)
+                m_program.lookarounds.push_back(static_cast<Lookaround>(to_underlying(Lookaround::CharacterClass) + character.value));
+            else if (character.op == ParsedRegex::CharacterType)
+                m_program.lookarounds.push_back(static_cast<Lookaround>(to_underlying(Lookaround::CharacterType) | character.value));
+            else
+                kak_assert(false);
+        }
         m_program.lookarounds.push_back(Lookaround::EndOfLookaround);
         return res;
     }
@@ -913,7 +904,7 @@ private:
     // Mutate start_desc with informations on which Codepoint could start a match.
     // Returns true if the node possibly does not consume the char, in which case
     // the next node would still be relevant for the parent node start chars computation.
-    template<MatchDirection direction>
+    template<RegexMode direction>
     bool compute_start_desc(ParsedRegex::NodeIndex index,
                              CompiledRegex::StartDesc& start_desc) const
     {
@@ -983,22 +974,21 @@ private:
             }
             case ParsedRegex::Sequence:
             {
-                bool did_not_consume = false;
-                auto does_not_consume = [&, this](auto child) {
-                    return this->compute_start_desc<direction>(child, start_desc);
-                };
-                did_not_consume = ForEachChild<direction>::apply(m_parsed_regex, index, does_not_consume);
-
-                return did_not_consume or node.quantifier.allows_none();
+                for (auto child : Children<direction>{m_parsed_regex, index})
+                {
+                    if (not compute_start_desc<direction>(child, start_desc))
+                        return node.quantifier.allows_none();
+                }
+                return true;
             }
             case ParsedRegex::Alternation:
             {
                 bool all_consumed = not node.quantifier.allows_none();
-                ForEachChild<>::apply(m_parsed_regex, index, [&](ParsedRegex::NodeIndex  child) {
+                for (auto child : Children<>{m_parsed_regex, index})
+                {
                     if (compute_start_desc<direction>(child, start_desc))
                         all_consumed = false;
-                    return true;
-                });
+                }
                 return not all_consumed;
             }
             case ParsedRegex::LineStart:
@@ -1017,7 +1007,7 @@ private:
         return false;
     }
 
-    template<MatchDirection direction>
+    template<RegexMode direction>
     [[gnu::noinline]]
     std::unique_ptr<CompiledRegex::StartDesc> compute_start_desc() const
     {
@@ -1135,9 +1125,6 @@ String dump_regex(const CompiledRegex& program)
                 res += format("{} ({})\n", name, str);
                 break;
             }
-            case CompiledRegex::FindNextStart:
-                res += "find next start\n";
-                break;
             case CompiledRegex::Match:
                 res += "match\n";
         }
@@ -1198,18 +1185,18 @@ bool is_ctype(CharacterType ctype, Codepoint cp)
 
 namespace
 {
-template<MatchDirection dir = MatchDirection::Forward>
-struct TestVM : CompiledRegex, ThreadedRegexVM<const char*, dir>
+template<RegexMode mode = RegexMode::Forward>
+struct TestVM : CompiledRegex, ThreadedRegexVM<const char*, mode>
 {
-    using VMType = ThreadedRegexVM<const char*, dir>;
+    using VMType = ThreadedRegexVM<const char*, mode>;
 
     TestVM(StringView re, bool dump = false)
-        : CompiledRegex{compile_regex(re, dir == MatchDirection::Forward ?
+        : CompiledRegex{compile_regex(re, mode & RegexMode::Forward ?
                                           RegexCompileFlags::None : RegexCompileFlags::Backward)},
           VMType{(const CompiledRegex&)*this}
     { if (dump) puts(dump_regex(*this).c_str()); }
 
-    bool exec(StringView re, RegexExecFlags flags = RegexExecFlags::AnyMatch)
+    bool exec(StringView re, RegexExecFlags flags = RegexExecFlags::None)
     {
         return VMType::exec(re.begin(), re.end(), re.begin(), re.end(), flags);
     }
@@ -1293,11 +1280,11 @@ auto test_regex = UnitTest{[]{
     }
 
     {
-        TestVM<> vm{R"(f.*a(.*o))"};
-        kak_assert(vm.exec("blahfoobarfoobaz", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"(f.*a(.*o))"};
+        kak_assert(vm.exec("blahfoobarfoobaz"));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]} == "foobarfoo");
         kak_assert(StringView{vm.captures()[2], vm.captures()[3]} == "rfoo");
-        kak_assert(vm.exec("mais que fais la police", RegexExecFlags::Search));
+        kak_assert(vm.exec("mais que fais la police"));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]} == "fais la po");
         kak_assert(StringView{vm.captures()[2], vm.captures()[3]} == " po");
     }
@@ -1357,21 +1344,21 @@ auto test_regex = UnitTest{[]{
     }
 
     {
-        TestVM<> vm{R"(foo\Kbar)"};
-        kak_assert(vm.exec("foobar", RegexExecFlags::None));
+        TestVM<RegexMode::Forward> vm{R"(foo\Kbar)"};
+        kak_assert(vm.exec("foobar"));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]} == "bar");
-        kak_assert(not vm.exec("bar", RegexExecFlags::None));
+        kak_assert(not vm.exec("bar"));
     }
 
     {
-        TestVM<> vm{R"((fo+?).*)"};
-        kak_assert(vm.exec("foooo", RegexExecFlags::None));
+        TestVM<RegexMode::Forward> vm{R"((fo+?).*)"};
+        kak_assert(vm.exec("foooo"));
         kak_assert(StringView{vm.captures()[2], vm.captures()[3]} == "fo");
     }
 
     {
-        TestVM<> vm{R"((?=fo[\w]).)"};
-        kak_assert(vm.exec("barfoo", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"((?=fo[\w]).)"};
+        kak_assert(vm.exec("barfoo"));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]} == "f");
     }
 
@@ -1422,77 +1409,83 @@ auto test_regex = UnitTest{[]{
     }
 
     {
-        TestVM<> vm{R"((?<!\\)(?:\\\\)*")"};
-        kak_assert(vm.exec("foo\"", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"((?<!\\)(?:\\\\)*")"};
+        kak_assert(vm.exec("foo\""));
     }
 
     {
-        TestVM<> vm{R"($)"};
-        kak_assert(vm.exec("foo\n", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"($)"};
+        kak_assert(vm.exec("foo\n"));
         kak_assert(*vm.captures()[0] == '\n');
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"(fo{1,})"};
-        kak_assert(vm.exec("foo1fooo2", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"(fo{1,})"};
+        kak_assert(vm.exec("foo1fooo2"));
         kak_assert(*vm.captures()[1] == '2');
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"((?<=f)oo(b[ae]r)?(?=baz))"};
-        kak_assert(vm.exec("foobarbazfoobazfooberbaz", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"((?<=f)oo(b[ae]r)?(?=baz))"};
+        kak_assert(vm.exec("foobarbazfoobazfooberbaz"));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]}  == "oober");
         kak_assert(StringView{vm.captures()[2], vm.captures()[3]}  == "ber");
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"((baz|boz|foo|qux)(?<!baz)(?<!o))"};
-        kak_assert(vm.exec("quxbozfoobaz", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"((baz|boz|foo|qux)(?<!baz)(?<!o))"};
+        kak_assert(vm.exec("quxbozfoobaz"));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]}  == "boz");
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"(foo)"};
-        kak_assert(vm.exec("foofoo", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"(foo)"};
+        kak_assert(vm.exec("foofoo"));
         kak_assert(*vm.captures()[1]  == 0);
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"($)"};
-        kak_assert(vm.exec("foo\nbar\nbaz\nqux", RegexExecFlags::Search | RegexExecFlags::NotEndOfLine));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"($)"};
+        kak_assert(vm.exec("foo\nbar\nbaz\nqux", RegexExecFlags::NotEndOfLine));
         kak_assert(StringView{vm.captures()[0]}  == "\nqux");
-        kak_assert(vm.exec("foo\nbar\nbaz\nqux", RegexExecFlags::Search));
+        kak_assert(vm.exec("foo\nbar\nbaz\nqux", RegexExecFlags::None));
         kak_assert(StringView{vm.captures()[0]}  == "");
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"(^)"};
-        kak_assert(not vm.exec("foo", RegexExecFlags::Search | RegexExecFlags::NotBeginOfLine));
-        kak_assert(vm.exec("foo", RegexExecFlags::Search));
-        kak_assert(vm.exec("foo\nbar", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"(^)"};
+        kak_assert(not vm.exec("foo", RegexExecFlags::NotBeginOfLine));
+        kak_assert(vm.exec("foo", RegexExecFlags::None));
+        kak_assert(vm.exec("foo\nbar", RegexExecFlags::None));
         kak_assert(StringView{vm.captures()[0]}  == "bar");
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"(\A\w+)"};
-        kak_assert(vm.exec("foo\nbar\nbaz", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"(\A\w+)"};
+        kak_assert(vm.exec("foo\nbar\nbaz", RegexExecFlags::None));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]}  == "foo");
     }
 
     {
-        TestVM<MatchDirection::Backward> vm{R"(\b\w+\z)"};
-        kak_assert(vm.exec("foo\nbar\nbaz", RegexExecFlags::Search));
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"(\b\w+\z)"};
+        kak_assert(vm.exec("foo\nbar\nbaz", RegexExecFlags::None));
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]}  == "baz");
     }
 
+    {
+        TestVM<RegexMode::Backward | RegexMode::Search> vm{R"(a[^\n]*\n|\n)"};
+        kak_assert(vm.exec("foo\nbar\nb", RegexExecFlags::None));
+        kak_assert(StringView{vm.captures()[0], vm.captures()[1]}  == "ar\n");
+    }
+
     {
         TestVM<> vm{R"(()*)"};
         kak_assert(not vm.exec(" "));
     }
 
     {
-        TestVM<> vm{R"(\b(?<!-)(a|b|)(?!-)\b)"};
-        kak_assert(vm.exec("# foo bar", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"(\b(?<!-)(a|b|)(?!-)\b)"};
+        kak_assert(vm.exec("# foo bar", RegexExecFlags::None));
         kak_assert(*vm.captures()[0] == '#');
     }
 
@@ -1502,19 +1495,19 @@ auto test_regex = UnitTest{[]{
     }
 
     {
-        TestVM<> vm{R"((?i)FOO)"};
-        kak_assert(vm.exec("foo", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"((?i)FOO)"};
+        kak_assert(vm.exec("foo", RegexExecFlags::None));
     }
 
     {
-        TestVM<> vm{R"(.?(?=foo))"};
-        kak_assert(vm.exec("afoo", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"(.?(?=foo))"};
+        kak_assert(vm.exec("afoo", RegexExecFlags::None));
         kak_assert(*vm.captures()[0] == 'a');
     }
 
     {
-        TestVM<> vm{R"((?i)(?=Foo))"};
-        kak_assert(vm.exec("fOO", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"((?i)(?=Foo))"};
+        kak_assert(vm.exec("fOO", RegexExecFlags::None));
         kak_assert(*vm.captures()[0] == 'f');
     }
 
@@ -1529,8 +1522,8 @@ auto test_regex = UnitTest{[]{
     }
 
     {
-        TestVM<> vm{R"(д)"};
-        kak_assert(vm.exec("д", RegexExecFlags::Search));
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"(д)"};
+        kak_assert(vm.exec("д", RegexExecFlags::None));
     }
 
     {
