@@ -19,6 +19,7 @@
 #include <string.h>
 #include <pwd.h>
 #include <fcntl.h>
+#include <errno.h>
 
 
 namespace Kakoune
@@ -49,8 +50,8 @@ public:
     MsgWriter(RemoteBuffer& buffer, MessageType type)
         : m_buffer{buffer}, m_start{(uint32_t)buffer.size()}
     {
-        write(type);
-        write((uint32_t)0); // message size, to be patched on write
+        write_field(type);
+        write_field((uint32_t)0); // message size, to be patched on write
     }
 
     ~MsgWriter()
@@ -59,87 +60,94 @@ public:
         memcpy(m_buffer.data() + m_start + sizeof(MessageType), &count, sizeof(uint32_t));
     }
 
-    void write(const char* val, size_t size)
+    template<typename ...Args>
+    void write(Args&&... args)
+    {
+        (write_field(std::forward<Args>(args)), ...);
+    }
+
+private:
+    void write_raw(const char* val, size_t size)
     {
         m_buffer.insert(m_buffer.end(), val, val + size);
     }
 
     template<typename T>
-    void write(const T& val)
+    void write_field(const T& val)
     {
         static_assert(std::is_trivially_copyable<T>::value, "");
-        write((const char*)&val, sizeof(val));
+        write_raw((const char*)&val, sizeof(val));
     }
 
-    void write(StringView str)
+    void write_field(StringView str)
     {
-        write(str.length());
-        write(str.data(), (int)str.length());
+        write_field(str.length());
+        write_raw(str.data(), (int)str.length());
     };
 
-    void write(const String& str)
+    void write_field(const String& str)
     {
-        write(StringView{str});
+        write_field(StringView{str});
     }
 
     template<typename T>
-    void write(ConstArrayView<T> view)
+    void write_field(ConstArrayView<T> view)
     {
-        write<uint32_t>(view.size());
+        write_field<uint32_t>(view.size());
         for (auto& val : view)
-            write(val);
+            write_field(val);
     }
 
     template<typename T, MemoryDomain domain>
-    void write(const Vector<T, domain>& vec)
+    void write_field(const Vector<T, domain>& vec)
     {
-        write(ConstArrayView<T>(vec));
+        write_field(ConstArrayView<T>(vec));
     }
 
     template<typename Key, typename Val, MemoryDomain domain>
-    void write(const HashMap<Key, Val, domain>& map)
+    void write_field(const HashMap<Key, Val, domain>& map)
     {
-        write<uint32_t>(map.size());
+        write_field<uint32_t>(map.size());
         for (auto& val : map)
         {
-            write(val.key);
-            write(val.value);
+            write_field(val.key);
+            write_field(val.value);
         }
     }
 
     template<typename T>
-    void write(const Optional<T>& val)
+    void write_field(const Optional<T>& val)
     {
-        write((bool)val);
+        write_field((bool)val);
         if (val)
-            write(*val);
+            write_field(*val);
     }
 
-    void write(Color color)
+    void write_field(Color color)
     {
-        write(color.color);
+        write_field(color.color);
         if (color.color == Color::RGB)
         {
-            write(color.r);
-            write(color.g);
-            write(color.b);
+            write_field(color.r);
+            write_field(color.g);
+            write_field(color.b);
         }
     }
 
-    void write(const DisplayAtom& atom)
+    void write_field(const DisplayAtom& atom)
     {
-        write(atom.content());
-        write(atom.face);
+        write_field(atom.content());
+        write_field(atom.face);
     }
 
-    void write(const DisplayLine& line)
+    void write_field(const DisplayLine& line)
     {
-        write(line.atoms());
+        write_field(line.atoms());
     }
 
-    void write(const DisplayBuffer& display_buffer)
+    void write_field(const DisplayBuffer& display_buffer)
     {
-        write(display_buffer.lines());
+        write_field(display_buffer.lines());
     }
 
 private:
@@ -149,6 +157,58 @@ private:
 
 class MsgReader
 {
+private:
+    template<typename T>
+    struct Reader {
+        static T read(MsgReader& reader)
+        {
+            static_assert(std::is_trivially_copyable<T>::value, "");
+            T res;
+            reader.read(reinterpret_cast<char*>(&res), sizeof(T));
+            return res;
+        }
+    };
+
+    template<typename T, MemoryDomain domain>
+    struct Reader<Vector<T,domain>> {
+        static Vector<T, domain> read(MsgReader& reader)
+        {
+            uint32_t size = Reader<uint32_t>::read(reader);
+            Vector<T,domain> res;
+            res.reserve(size);
+            while (size--)
+                res.push_back(std::move(Reader<T>::read(reader)));
+            return res;
+        }
+    };
+
+    template<typename Key, typename Value, MemoryDomain domain>
+    struct Reader<HashMap<Key, Value, domain>> {
+        static HashMap<Key, Value, domain> read(MsgReader& reader)
+        {
+            uint32_t size = Reader<uint32_t>::read(reader);
+            HashMap<Key, Value, domain> res;
+            res.reserve(size);
+            while (size--)
+            {
+                auto key = Reader<Key>::read(reader);
+                auto val = Reader<Value>::read(reader);
+                res.insert({std::move(key), std::move(val)});
+            }
+            return res;
+        }
+    };
+
+    template<typename T>
+    struct Reader<Optional<T>> {
+        static Optional<T> read(MsgReader& reader)
+        {
+            if (not Reader<bool>::read(reader))
+                return {};
+            return Reader<T>::read(reader);
+        }
+    };
+
 public:
     void read_available(int sock)
     {
@@ -197,44 +257,7 @@ public:
     template<typename T>
     T read()
     {
-        static_assert(std::is_trivially_copyable<T>::value, "");
-        T res;
-        read(reinterpret_cast<char*>(&res), sizeof(T));
-        return res;
-    }
-
-    template<typename T>
-    Vector<T> read_vector()
-    {
-        uint32_t size = read<uint32_t>();
-        Vector<T> res;
-        res.reserve(size);
-        while (size--)
-            res.push_back(read<T>());
-        return res;
-    }
-
-    template<typename Key, typename Val, MemoryDomain domain>
-    HashMap<Key, Val, domain> read_hash_map()
-    {
-        uint32_t size = read<uint32_t>();
-        HashMap<Key, Val, domain> res;
-        res.reserve(size);
-        while (size--)
-        {
-            auto key = read<Key>();
-            auto val = read<Val>();
-            res.insert({std::move(key), std::move(val)});
-        }
-        return res;
-    }
-
-    template<typename T>
-    Optional<T> read_optional()
-    {
-        if (not read<bool>())
-            return {};
-        return read<T>();
+        return Reader<T>::read(*this);
     }
 
     void reset()
@@ -261,52 +284,62 @@ private:
 };
 
 template<>
-String MsgReader::read<String>()
-{
-    ByteCount length = read<ByteCount>();
-    String res;
-    if (length > 0)
+struct MsgReader::Reader<String> {
+    static String read(MsgReader& reader)
     {
-        res.force_size((int)length);
-        read(&res[0_byte], (int)length);
+        ByteCount length = Reader<ByteCount>::read(reader);
+        String res;
+        if (length > 0)
+        {
+            res.force_size((int)length);
+            reader.read(&res[0_byte], (int)length);
+        }
+        return res;
     }
-    return res;
-}
+};
 
 template<>
-Color MsgReader::read<Color>()
-{
-    Color res;
-    res.color = read<Color::NamedColor>();
-    if (res.color == Color::RGB)
+struct MsgReader::Reader<Color> {
+    static Color read(MsgReader& reader)
     {
-        res.r = read<unsigned char>();
-        res.g = read<unsigned char>();
-        res.b = read<unsigned char>();
+        Color res;
+        res.color = Reader<Color::NamedColor>::read(reader);
+        if (res.color == Color::RGB)
+        {
+            res.r = Reader<unsigned char>::read(reader);
+            res.g = Reader<unsigned char>::read(reader);
+            res.b = Reader<unsigned char>::read(reader);
+        }
+        return res;
     }
-    return res;
-}
+};
 
 template<>
-DisplayAtom MsgReader::read<DisplayAtom>()
-{
-    String content = read<String>();
-    return {std::move(content), read<Face>()};
-}
+struct MsgReader::Reader<DisplayAtom> {
+    static DisplayAtom read(MsgReader& reader)
+    {
+        String content = Reader<String>::read(reader);
+        return {std::move(content), Reader<Face>::read(reader)};
+    }
+};
 
 template<>
-DisplayLine MsgReader::read<DisplayLine>()
-{
-    return {read_vector<DisplayAtom>()};
-}
+struct MsgReader::Reader<DisplayLine> {
+    static DisplayLine read(MsgReader& reader)
+    {
+        return {Reader<Vector<DisplayAtom>>::read(reader)};
+    }
+};
 
 template<>
-DisplayBuffer MsgReader::read<DisplayBuffer>()
-{
-    DisplayBuffer db;
-    db.lines() = read_vector<DisplayLine>();
-    return db;
-}
+struct MsgReader::Reader<DisplayBuffer> {
+    static DisplayBuffer read(MsgReader& reader)
+    {
+        DisplayBuffer db;
+        db.lines() = Reader<Vector<DisplayLine>>::read(reader);
+        return db;
+    }
+};
 
 
 class RemoteUI : public UserInterface
@@ -349,6 +382,14 @@ public:
     void exit(int status);
 
 private:
+    template<typename ...Args>
+    void send_message(MessageType type, Args&&... args)
+    {
+        MsgWriter msg{m_send_buffer, type};
+        msg.write(std::forward<Args>(args)...);
+        m_socket_watcher.events() |= FdEvents::Write;
+    }
+
     FDWatcher     m_socket_watcher;
     MsgReader     m_reader;
     DisplayCoord  m_dimensions;
@@ -413,7 +454,8 @@ RemoteUI::~RemoteUI()
     // Try to send the remaining data if possible, as it might contain the desired exit status
     try
     {
-        send_data(m_socket_watcher.fd(), m_send_buffer);
+        if (m_socket_watcher.fd() != -1)
+            send_data(m_socket_watcher.fd(), m_send_buffer);
     }
     catch (disconnected&)
     {
@@ -427,96 +469,63 @@ void RemoteUI::menu_show(ConstArrayView<DisplayLine> choices,
                          DisplayCoord anchor, Face fg, Face bg,
                          MenuStyle style)
 {
-    MsgWriter msg{m_send_buffer, MessageType::MenuShow};
-    msg.write(choices);
-    msg.write(anchor);
-    msg.write(fg);
-    msg.write(bg);
-    msg.write(style);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::MenuShow, choices, anchor, fg, bg, style);
 }
 
 void RemoteUI::menu_select(int selected)
 {
-    MsgWriter msg{m_send_buffer, MessageType::MenuSelect};
-    msg.write(selected);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::MenuSelect, selected);
 }
 
 void RemoteUI::menu_hide()
 {
-    MsgWriter msg{m_send_buffer, MessageType::MenuHide};
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::MenuHide);
 }
 
 void RemoteUI::info_show(StringView title, StringView content,
                          DisplayCoord anchor, Face face,
                          InfoStyle style)
 {
-    MsgWriter msg{m_send_buffer, MessageType::InfoShow};
-    msg.write(title);
-    msg.write(content);
-    msg.write(anchor);
-    msg.write(face);
-    msg.write(style);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::InfoShow, title, content, anchor, face, style);
 }
 
 void RemoteUI::info_hide()
 {
-    MsgWriter msg{m_send_buffer, MessageType::InfoHide};
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::InfoHide);
 }
 
 void RemoteUI::draw(const DisplayBuffer& display_buffer,
                     const Face& default_face,
                     const Face& padding_face)
 {
-    MsgWriter msg{m_send_buffer, MessageType::Draw};
-    msg.write(display_buffer);
-    msg.write(default_face);
-    msg.write(padding_face);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::Draw, display_buffer, default_face, padding_face);
 }
 
 void RemoteUI::draw_status(const DisplayLine& status_line,
                            const DisplayLine& mode_line,
                            const Face& default_face)
 {
-    MsgWriter msg{m_send_buffer, MessageType::DrawStatus};
-    msg.write(status_line);
-    msg.write(mode_line);
-    msg.write(default_face);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::DrawStatus, status_line, mode_line, default_face);
 }
 
 void RemoteUI::set_cursor(CursorMode mode, DisplayCoord coord)
 {
-    MsgWriter msg{m_send_buffer, MessageType::SetCursor};
-    msg.write(mode);
-    msg.write(coord);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::SetCursor, mode, coord);
 }
 
 void RemoteUI::refresh(bool force)
 {
-    MsgWriter msg{m_send_buffer, MessageType::Refresh};
-    msg.write(force);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::Refresh, force);
 }
 
 void RemoteUI::set_ui_options(const Options& options)
 {
-    MsgWriter msg{m_send_buffer, MessageType::SetOptions};
-    msg.write(options);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::SetOptions, options);
 }
 
 void RemoteUI::exit(int status)
 {
-    MsgWriter msg{m_send_buffer, MessageType::Exit};
-    msg.write(status);
-    m_socket_watcher.events() |= FdEvents::Write;
+    send_message(MessageType::Exit, status);
 }
 
 String get_user_name()
@@ -569,12 +578,7 @@ RemoteClient::RemoteClient(StringView session, StringView name, std::unique_ptr<
 
     {
         MsgWriter msg{m_send_buffer, MessageType::Connect};
-        msg.write(pid);
-        msg.write(name);
-        msg.write(init_command);
-        msg.write(init_coord);
-        msg.write(m_ui->dimensions());
-        msg.write(env_vars);
+        msg.write(pid, name, init_command, init_coord, m_ui->dimensions(), env_vars);
     }
 
     m_ui->set_on_key([this](Key key){
@@ -602,7 +606,7 @@ RemoteClient::RemoteClient(StringView session, StringView name, std::unique_ptr<
             {
             case MessageType::MenuShow:
             {
-                auto choices = reader.read_vector<DisplayLine>();
+                auto choices = reader.read<Vector<DisplayLine>>();
                 auto anchor = reader.read<DisplayCoord>();
                 auto fg = reader.read<Face>();
                 auto bg = reader.read<Face>();
@@ -656,7 +660,7 @@ RemoteClient::RemoteClient(StringView session, StringView name, std::unique_ptr<
                 m_ui->refresh(reader.read<bool>());
                 break;
             case MessageType::SetOptions:
-                m_ui->set_ui_options(reader.read_hash_map<String, String, MemoryDomain::Options>());
+                m_ui->set_ui_options(reader.read<HashMap<String, String, MemoryDomain::Options>>());
                 break;
             case MessageType::Exit:
                 m_exit_status = reader.read<int>();
@@ -722,9 +726,9 @@ private:
                 auto pid = m_reader.read<int>();
                 auto name = m_reader.read<String>();
                 auto init_cmds = m_reader.read<String>();
-                auto init_coord = m_reader.read_optional<BufferCoord>();
+                auto init_coord = m_reader.read<Optional<BufferCoord>>();
                 auto dimensions = m_reader.read<DisplayCoord>();
-                auto env_vars = m_reader.read_hash_map<String, String, MemoryDomain::EnvVars>();
+                auto env_vars = m_reader.read<HashMap<String, String, MemoryDomain::EnvVars>>();
                 auto* ui = new RemoteUI{sock, dimensions};
                 ClientManager::instance().create_client(
                     std::unique_ptr<UserInterface>(ui), pid, std::move(name),
@@ -819,6 +823,9 @@ bool Server::rename_session(StringView name)
     String new_socket_file = format("{}/kakoune/{}/{}", tmpdir(),
                                     get_user_name(), name);
 
+    if (file_exists(new_socket_file))
+        return false;
+
     if (rename(old_socket_file.c_str(), new_socket_file.c_str()) != 0)
         return false;
 
