@@ -19,7 +19,7 @@ void GlobalScope::on_option_changed(const Option& option)
 {
     Context empty_context{Context::EmptyContextFlag{}};
     hooks().run_hook(Hook::GlobalSetOption,
-                     format("{}={}", option.name(), option.get_as_string(Quoting::Kakoune)),
+                     format("{}={}", option.name(), option.get_as_string(Quoting::Raw)),
                      empty_context);
 }
 
