@@ -8,7 +8,7 @@ namespace Kakoune
 {
 
 SelectionList::SelectionList(Buffer& buffer, Selection s, size_t timestamp)
-    : m_buffer(&buffer), m_selections({ std::move(s) }), m_timestamp(timestamp)
+    : m_selections({ std::move(s) }), m_buffer(&buffer), m_timestamp(timestamp)
 {
     check_invariant();
 }
@@ -17,7 +17,7 @@ SelectionList::SelectionList(Buffer& buffer, Selection s)
     : SelectionList(buffer, std::move(s), buffer.timestamp()) {}
 
 SelectionList::SelectionList(Buffer& buffer, Vector<Selection> list, size_t timestamp)
-    : m_buffer(&buffer), m_selections(std::move(list)), m_timestamp(timestamp)
+    : m_selections(std::move(list)), m_buffer(&buffer), m_timestamp(timestamp)
 {
     kak_assert(size() > 0);
     m_main = size() - 1;
@@ -28,7 +28,7 @@ SelectionList::SelectionList(Buffer& buffer, Vector<Selection> list)
     : SelectionList(buffer, std::move(list), buffer.timestamp()) {}
 
 SelectionList::SelectionList(SelectionList::UnsortedTag, Buffer& buffer, Vector<Selection> list, size_t timestamp, size_t main)
-    : m_buffer(&buffer), m_selections(std::move(list)), m_timestamp(timestamp)
+    : m_selections(std::move(list)), m_buffer(&buffer), m_timestamp(timestamp)
 {
     sort_and_merge_overlapping();
     check_invariant();
@@ -50,6 +50,12 @@ void SelectionList::set(Vector<Selection> list, size_t main)
     check_invariant();
 }
 
+bool compare_selections(const Selection& lhs, const Selection& rhs)
+{
+    const auto lmin = lhs.min(), rmin = rhs.min();
+    return lmin == rmin ? lhs.max() < rhs.max() : lmin < rmin;
+}
+
 namespace
 {
 
@@ -78,12 +84,6 @@ BufferCoord update_erase(BufferCoord coord, BufferCoord begin, BufferCoord end)
     return coord;
 } */
 
-bool compare_selections(const Selection& lhs, const Selection& rhs)
-{
-    const auto lmin = lhs.min(), rmin = rhs.min();
-    return lmin == rmin ? lhs.max() < rhs.max() : lmin < rmin;
-}
-
 template<typename Iterator, typename OverlapsFunc>
 Iterator merge_overlapping(Iterator begin, Iterator end, size_t& main, OverlapsFunc overlaps)
 {
@@ -218,7 +218,7 @@ void clamp_selections(Vector<Selection>& selections, const Buffer& buffer)
         clamp(sel, buffer);
 }
 
-void update_selections(Vector<Selection>& selections, size_t& main, Buffer& buffer, size_t timestamp)
+void update_selections(Vector<Selection>& selections, size_t& main, Buffer& buffer, size_t timestamp, bool merge)
 {
     if (timestamp == buffer.timestamp())
         return;
@@ -242,20 +242,22 @@ void update_selections(Vector<Selection>& selections, size_t& main, Buffer& buff
         }
         kak_assert(std::is_sorted(selections.begin(), selections.end(),
                                   compare_selections));
-        selections.erase(
-            merge_overlapping(selections.begin(), selections.end(),
-                              main, overlaps), selections.end());
+        if (merge)
+            selections.erase(
+                merge_overlapping(selections.begin(), selections.end(),
+                                  main, overlaps), selections.end());
     }
     for (auto& sel : selections)
         clamp(sel, buffer);
 
-    selections.erase(merge_overlapping(selections.begin(), selections.end(),
-                                       main, overlaps), selections.end());
+    if (merge)
+        selections.erase(merge_overlapping(selections.begin(), selections.end(),
+                                           main, overlaps), selections.end());
 }
 
-void SelectionList::update()
+void SelectionList::update(bool merge)
 {
-    update_selections(m_selections, m_main, *m_buffer, m_timestamp);
+    update_selections(m_selections, m_main, *m_buffer, m_timestamp, merge);
     check_invariant();
     m_timestamp = m_buffer->timestamp();
 }
@@ -409,8 +411,12 @@ void SelectionList::insert(ConstArrayView<String> strings, InsertMode mode,
         const String& str = strings[std::min(index, strings.size()-1)];
 
         const auto pos = (mode == InsertMode::Replace) ?
-            replace(*m_buffer, sel, str)
-          : m_buffer->insert(changes_tracker.get_new_coord(insert_pos[index]), str);
+            sel.min() : changes_tracker.get_new_coord(insert_pos[index]);
+
+        if (mode == InsertMode::Replace)
+            replace(*m_buffer, sel, str);
+        else
+            m_buffer->insert(pos, str);
 
         size_t old_timestamp = m_timestamp;
         changes_tracker.update(*m_buffer, m_timestamp);
@@ -512,14 +518,14 @@ Selection selection_from_string(StringView desc)
     return Selection{anchor, cursor};
 }
 
-SelectionList selection_list_from_string(Buffer& buffer, ConstArrayView<String> descs)
+SelectionList selection_list_from_string(Buffer& buffer, ConstArrayView<String> descs, size_t timestamp)
 {
     if (descs.empty())
         throw runtime_error{"empty selection description"};
 
     auto sels = descs | transform([&](auto&& d) { auto s = selection_from_string(d); clamp(s, buffer); return s; })
                       | gather<Vector<Selection>>();
-    return {SelectionList::UnsortedTag{}, buffer, std::move(sels), buffer.timestamp(), 0};
+    return {SelectionList::UnsortedTag{}, buffer, std::move(sels), timestamp, 0};
 }
 
 }
